/* 
 *  Copyright (c) 2001 Dan Gudmundsson
 *  See the file "license.terms" for information on usage and redistribution
 *  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 *     $Id: esdl.h,v 1.5 2003/01/13 14:37:12 dgud Exp $
 */
/* Define function's */
#ifdef __cplusplus
    extern "C" {
#endif 


#ifndef SDL_H

#ifdef WIN32
#include <windows.h>  /* needed by Windows' gl.h etc */
#include <SDL.h>
#else
#ifdef __FreeBSD__
#include <SDL.h>
#else
#include <SDL/SDL.h>
#endif
#endif


#ifdef _OSX_COCOA
#include <OpenGL/gl.h>
#else
#include <GL/gl.h>
#endif

#include "esdl_conv.h"

#define MAXBUFF 8000000  /* Covers 1600x1200x4 (32bits) */
#define NOFFMEMREFS 512

#define error() {fprintf(stderr, "Error in %s:%d \n\r", \
                                 __FILE__, __LINE__); \
                return -1;}


typedef struct
{
   GLenum type;
   GLuint size;
   GLvoid *ptr;
} oglmem;

typedef struct sdl_data_def *sdl_data_ptr;
typedef int (*sdl_fun)(sdl_data_ptr, int, char*);

typedef struct sdl_code_fn_def {
   int op;
   char *str;
   sdl_fun fn;
} sdl_code_fn;

typedef sdl_fun (*sdl_load_fun)(void);


typedef struct sdl_data_def {
   void * driver_data;         /* Port or Driver specific data */
   sdl_code_fn ** fns;         /* Function definition table */

   /* GLOBAL ref table for getPointerv */
   oglmem *memRefs[NOFFMEMREFS];
   int noMemRefs;

   int op;                     /* Current (or last) function */
   int len;                    /* Length of message buffer */
   void * buff;                /* Pointer to message buffer */
   
   int debug;                  /* Debug level */

#ifdef _OSX_COCOA

  void * release_pool;
  void *app;
  //  void *thr;
  //  NSGraphicsContext *context;

#endif

} sdl_data;

#define ARGS(BUFF) ((buff)+2)

int do_command(sdl_data *, unsigned int, int, char*);
int sdl_send_erl(sdl_data *, int, void *);
int sdl_send(sdl_data *, int, char *);  /* should be removed */
char * sdl_getbuff(sdl_data *, int, char *);
sdl_code_fn ** init_fps();

/* int  readstream(char *buff); */
/* int  writestream(const int len, const char * buff); */

/*   These must exactly match those in src/esdl.hrl */
#define SDL_H                20
#define VIDEO_H              30
#define EVENTS_H            100
#define MOUSE_H             110
#define KEYBOARD_H          120
#define ACTIVE_H            130
#define JOYSTICK_H          133
#define AUDIO_H             150
#define SDL_UTIL_H          180
#define OPENGL_H            200
#define OPENGLU_H           600
#define OPENGL_EXTS_H       700  /* Must be last */
#define MAX_FUNCTIONS_H    1023  /* Current Max.. Increase if needed */

#define SDL_InitFunc (SDL_H + 1)
#define SDL_QuitFunc (SDL_InitFunc + 1)
#define SDL_GetErrorFunc (SDL_QuitFunc + 1)

#include "esdl_video.h"
#include "esdl_events.h"
#include "esdl_audio.h"
#include "esdl_util.h"

int es_init(sdl_data *sd, int len, char * buff);
int es_quit(sdl_data *sd, int len, char * buff);
int es_getError(sdl_data *sd, int len, char *buff);

#endif

#ifdef __cplusplus
    }
#endif 
