/*  
 *  Copyright (c) 2001 Dan Gudmundsson
 *  See the file "license.terms" for information on usage and redistribution
 *  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 * 
 *     $Id: esdl_audio.c,v 1.2 2001/11/22 15:09:10 dgud Exp $
 */
/* 
   Map erl esdl_events calls to C sdl calls      
*/

#include "esdl.h"
#include <string.h>
#include <stdlib.h>

struct {
   Uint8   *sound;		/* Pointer to wave data */
   Uint32  soundlen;		/* Length of wave data */
   int     repeat;              /* Play sample 'repeat' times */
   int     soundpos;		/* Current play position */
    
   Uint8 silence;
} wave;

int play_audio(sdl_data *sd, int len, char *buff) 
{
   char *bp, *start;
   int sendlen;
   oglmem * sbuff;
   bp = ARGS(buff);
   
   SDL_LockAudio();   
   
   sbuff = getPointer(bp);
   wave.sound    = sbuff->ptr;
   wave.soundlen = get32be(bp);
   wave.repeat   = get32be(bp);
   wave.soundpos = 0;
   
   SDL_UnlockAudio();
   bp = start = sdl_getbuff(sd, 0, buff);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
	error();           
     
   return 0;
}

void myaudiomixer(void *mydata, Uint8 *stream, int len)
{   
   Uint8 *waveptr;
   int    waveleft;
   
   if(wave.sound != NULL && wave.repeat != 0) 
   {
      /* Set up the pointers */
      waveptr  = wave.sound + wave.soundpos;
      waveleft = wave.soundlen - wave.soundpos;
      /* fprintf(stderr, "ED1 0x%X %d 0x%X %d repeat %d\n\r", 
	 stream, len, waveptr, waveleft, wave.repeat); */
	 /* Go! */
      while( waveleft < len ) {
	 memcpy(stream, waveptr, waveleft);
	 /*	 SDL_MixAudio(stream, waveptr, waveleft, SDL_MIX_MAXVOLUME); */
	 stream += waveleft;
	 len -= waveleft;
	 waveptr = wave.sound;
	 waveleft = wave.soundlen;
	 wave.soundpos = 0;
	 if((wave.repeat -= 1) == 0) {
	    memset(stream, wave.silence, len);
	    break;
	 }
      }
      if(wave.repeat != 0) {
	 /*	 fprintf(stderr, "ED2 0x%X %d 0x%X %d repeat %d\n\r", 
		 stream, len, waveptr, waveleft, wave.repeat); */
	 memcpy(stream, waveptr, len);
	 /*	 SDL_MixAudio(stream, waveptr, len, SDL_MIX_MAXVOLUME); */
	 wave.soundpos += len;    
      }
   }
   else { 
       memset(stream, wave.silence, len);
   }
}

/* API */
int es_audioDriverName(sdl_data *sd, int len, char *buff)
{
    int sendlen;
    char *bp, *start;
    
    bp = start = sdl_getbuff(sd, 256, buff);
    if(NULL != SDL_AudioDriverName(bp, 256)) {
       while(*bp != '\0')
	  bp++;
    }
    sendlen = bp - start;
    if(sdl_send(sd, sendlen, buff) != sendlen)
	error();    
    return 0;   
}

int es_openAudio(sdl_data *sd, int len, char *buff)
{
   int sendlen;
   char *bp, *start;
   int ff;
   SDL_AudioSpec desired, obtained, *obptr;
   bp = ARGS(buff);
   ff = get8(bp);
   desired.freq     = get32be(bp);
   desired.format   = get16be(bp);
   desired.channels = get8(bp);
   desired.samples  = get16be(bp);
   desired.padding  = get16be(bp);
   desired.callback = myaudiomixer;

   /* Init the global data structures */
   wave.sound = NULL;
   wave.soundpos = 0;
   wave.soundlen = 0;

   if(ff == 1)  /* Force the requested format */
      obptr = NULL;
   else 
      obptr = &obtained;
   
   bp = start = sdl_getbuff(sd, 16, buff);
   if( SDL_OpenAudio(&desired, obptr) < 0 ) {
      fprintf(stderr, "Couldn't open audio: %s\n", SDL_GetError());		
   } else {
      if(ff == 1) 
	  obptr = &desired;
       put32be(bp, obptr->freq);
       put16be(bp, obptr->format);
       put8(bp, obptr->channels);
       put8(bp, obptr->silence);
       put16be(bp, obptr->samples);
       put16be(bp, obptr->padding);
       put32be(bp, obptr->size);
       wave.silence = obptr->silence;
   } 
  
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
       error();       
   return 0;
}

int es_getAudioStatus(sdl_data *sd, int len, char *buff)
{
   int sendlen;
   char *bp, *start;
   SDL_audiostatus s;
   
   s = SDL_GetAudioStatus();
   start = bp = sdl_getbuff(sd, 1, buff);
   put8(bp, s);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
       error();       
   return 0;    
}

int es_pauseAudio(sdl_data *sd, int len, char *buff)
{
    char *bp;
    int pause;
    bp = ARGS(buff);
    pause = get8(bp);
    SDL_PauseAudio(pause);
    return 0;
}

int es_loadWAV(sdl_data *sd, int len, char *buff)
{
    int sendlen;
    char *name, *bp, *start;
    SDL_AudioSpec obtained;
    Uint8 * ptr;
    oglmem * ref =NULL;
    Uint32 blen;
    bp = ARGS(buff);
    buff[len] = '\0';
    name = bp;
    bp = start = sdl_getbuff(sd, 24, buff);
    if(NULL != SDL_LoadWAV(name, &obtained, &ptr, &blen)) {
       put32be(bp, obtained.freq);
       put16be(bp, obtained.format);
       put8(bp, obtained.channels);
       put8(bp, obtained.silence);
       put16be(bp, obtained.samples);
       put16be(bp, obtained.padding);
       put32be(bp, obtained.size);
       
       if((ref = insert_mem(sd, blen, GL_UNSIGNED_BYTE, ptr)) != NULL) {
	  putPointer(bp, ref);
	  put32be(bp, blen);
       }
    }
    sendlen = bp - start;
    if(sdl_send(sd, sendlen, buff) != sendlen)
	error();           
    return 0;
}

int es_loadWAVRW(sdl_data *sd, int len, char *buff)
{
    error();
}

int es_freeWAV(sdl_data *sd, int len, char *buff)
{
   char *bp;
   oglmem *ptr;
   bp = ARGS(buff);
   ptr = getPointer(bp);
   SDL_FreeWAV(ptr->ptr);
   ptr->ptr = NULL;
   free_mem(sd, ptr);
   return 0;    
}

int es_buildAudioCVT(sdl_data *sd, int len, char *buff)
{
   error();
}
int es_convertAudio(sdl_data *sd, int len, char *buff)
{
   char *bp, *start;
   oglmem *mptr, *nref;
   Uint16 oformat, nformat;
   Uint8  ochannels, nchannels;
   int    ofreq, nfreq, osize, nsize;
   SDL_AudioCVT  wav_cvt;
   int sendlen;

   bp = ARGS(buff);
   oformat = get16be(bp);
   ochannels = get8(bp);
   ofreq = get32be(bp);
   nformat = get16be(bp);
   nchannels = get8(bp);
   nfreq = get32be(bp);
   mptr = getPointer(bp);
   osize = get32be(bp);

   bp = start = sdl_getbuff(sd, 8, buff);
   
   /* Build AudioCVT */
   if(SDL_BuildAudioCVT(&wav_cvt,oformat, ochannels, ofreq,
			nformat, nchannels, nfreq) >= 0) {      
      /* Setup for conversion */
      nsize = osize*wav_cvt.len_mult;      
      wav_cvt.buf=(Uint8 *)malloc(nsize);
      if(wav_cvt.buf != NULL)
      {
	 wav_cvt.len=osize;
	 memcpy(wav_cvt.buf, mptr->ptr, osize);
	 if(SDL_ConvertAudio(&wav_cvt) >= 0)
	 {
	    if((nref = insert_mem(sd, nsize, GL_UNSIGNED_BYTE, wav_cvt.buf)) != NULL) {
	       putPointer(bp, nref);
	       put32be(bp, nsize);	 	
	    } 
	 }
      }
   }   
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();           
   return 0;   
}
int es_mixAudio(sdl_data *sd, int len, char *buff)
{

   error();
}
int es_lockAudio(sdl_data *sd, int len, char *buff)
{

   error();
}
int es_unlockAudio(sdl_data *sd, int len, char *buff)
{

   error();
}

int es_closeAudio(sdl_data *sd, int len, char *buff)
{
    SDL_CloseAudio(); 
    return 0;
}
