/*
 *  Copyright (c) 2001 Dan Gudmundsson
 *  See the file "license.terms" for information on usage and redistribution
 *  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * 
 *     $Id: esdl_events.c,v 1.5 2002/11/28 20:36:14 bjorng Exp $
 */
/* 
   Map erl esdl_events calls to C sdl calls      
*/

#include "esdl.h"
#ifdef WIN32
#include <SDL_events.h>
#else
#ifdef __FreeBSD__
#include <SDL_events.h>
#else
#include <SDL/SDL_events.h>
#endif
#endif

/* Foward decls */
char * encode_event(const SDL_Event * ev, char * bp);

/* API */
int es_pumpEvents(sdl_data *sd, int len,char *buff)
{   
   SDL_PumpEvents();
   return 1;
}

int es_peepEvents(sdl_data *sd, int len,char *buff)
{
   SDL_Event events[256];
   int numevents, res, sendlen, i;
   SDL_eventaction action;
   Uint32 mask;
   char *bp, *start;
   
   bp = ARGS(buff);
   numevents = get8(bp);
   action = get8(bp);
   mask  = get32be(bp);
   SDL_PumpEvents();
   res = SDL_PeepEvents(events, numevents, action, mask);
   /*   fprintf(stderr, "%d", res); */
   bp = start = sdl_getbuff(sd, 1+res*sizeof(SDL_Event), buff);
   put8(bp, res);
   for(i = 0; i < res; i++) 
       bp = encode_event(&(events[i]), bp);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;   
}

int es_pollEvent(sdl_data *sd, int len,char *buff)
{
    SDL_Event event;
    int sendlen;
    char *bp, *start;
    
    bp = start = sdl_getbuff(sd, sizeof(SDL_Event), buff);
    if(SDL_PollEvent(&event)) 
       bp = encode_event(&event, bp);
    else
       put8(bp, SDL_NOEVENT);    
    sendlen = bp - start;
    if(sdl_send(sd, sendlen, buff) != sendlen)
	error();    
    return 0;   
}

int es_waitEvent(sdl_data *sd, int len,char *buff)
{
    SDL_Event event;
    int sendlen;
    char *bp, *start;
    
    bp = start = sdl_getbuff(sd, sizeof(SDL_Event), buff);
    SDL_WaitEvent(&event);
    bp = encode_event(&event, bp);
    sendlen = bp - start;
    if(sdl_send(sd, sendlen, buff) != sendlen)
	error();    
    return 0;   
}

int es_pushEvent(sdl_data *sd, int len,char *buff)
{
    error(); /* Not implemented */
}

int es_eventState(sdl_data *sd, int len,char *buff) 
{
    Uint8 type, res;
    int sendlen, state;
    char *bp, *start;
    
    bp = ARGS(buff);
    type  = get8(bp);
    state = get8(bp);

    res = SDL_EventState(type, state);
    bp = start = sdl_getbuff(sd, 1, buff);
    put8(bp, res);
    sendlen = bp - start;
    if(sdl_send(sd, sendlen, buff) != sendlen)
       error();    
    return 0;       
}

/* API from the other files e.g. mouse keyboard active */ 

int es_getAppState(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   Uint8 state;

   bp = start = sdl_getbuff(sd, 1, buff);
   state = SDL_GetAppState();
   put8(bp, state);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         
}
 
int es_enableUNICODE(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   Uint8 enable;

   bp = ARGS(buff);
   enable = get8(bp);
   bp = start = sdl_getbuff(sd, 1, buff);
   enable = SDL_EnableUNICODE(enable);
   put8(bp, enable);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         
}

int es_enableKeyRepeat(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   Uint16 delay, intv, res;
   bp = ARGS(buff);
   delay = get16be(bp);
   intv  = get16be(bp);
   bp = start = sdl_getbuff(sd, 1, buff);
   res = SDL_EnableKeyRepeat(delay, intv);
   put8(bp, res);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;               
}

int es_getKeyName(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start, *name;
   Uint16 key;
   
   bp = ARGS(buff);
   key = get16be(bp);
   bp = start = sdl_getbuff(sd, 128, buff);
   name = SDL_GetKeyName(key);
   while(*name != '\0') {
      put8(bp, *name);
      name++;
   }
   
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;               
}

int es_getKeyState(sdl_data *sd, int len,char *buff)
{
   Uint8 * keys;
   char *bp, *start;
   int length, sendlen, i;

   keys = SDL_GetKeyState(&length);
   bp = start = sdl_getbuff(sd, length, buff);
   for(i=0; i<length; i++)
      put8(bp, keys[i]);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         
}

int es_getModState(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   Uint16 state;

   bp = start = sdl_getbuff(sd, 2, buff);
   state = SDL_GetModState();
   put16be(bp, state);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         
}

int es_setModState(sdl_data *sd, int len,char *buff)
{
   char *bp;
   Uint16 state;

   bp = ARGS(buff);
   state = get16be(bp);
   SDL_SetModState(state);
   return 0;         
}

int es_getMouseState(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   Uint8 state;
   int x, y;

   bp = start = sdl_getbuff(sd, 5, buff);
   state = SDL_GetMouseState(&x, &y);
   put8(bp, state);
   put16be(bp, x);
   put16be(bp, y);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         
}

int es_getRelativeMouseState(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   Uint8 state;
   int x, y;

   bp = start = sdl_getbuff(sd, 5, buff);
   state = SDL_GetMouseState(&x, &y);
   put8(bp, state);
   put16be(bp, x);
   put16be(bp, y);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         
}

int es_warpMouse(sdl_data *sd, int len,char *buff)
{
   char *bp;
   int x, y;
   
   bp = ARGS(buff);
   x = get16be(bp);
   y = get16be(bp);
   SDL_WarpMouse(x, y);
   return 0;
}

int es_createCursor(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   Uint8 data[32*32], mask[32*32];
   int w,h, hotx, hoty, i, ds, ms;
   SDL_Cursor * cursor;
   bp = ARGS(buff);
   ds = get16be(bp);
   for(i = 0; i < ds && (i < (32*32)); i++)
      data[i] = get8(bp);
   ms = get16be(bp);
   for(i = 0; i < ds && (i < (32*32)); i++)
      mask[i] = get8(bp);
   w = get16be(bp);
   h = get16be(bp);
   hotx = get16be(bp);
   hoty = get16be(bp);

   cursor = SDL_CreateCursor(data, mask, w, h, hotx, hoty);
   bp = start = sdl_getbuff(sd, 4, buff);
   put32be(bp, (int) cursor);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;
}

int es_setCursor(sdl_data *sd, int len,char *buff)
{
   char *bp;
   SDL_Cursor *c;
   
   bp = ARGS(buff);
   c = (SDL_Cursor *) get32be(bp);
   SDL_SetCursor(c);
   return 0;
}

int es_getCursor(sdl_data *sd, int len,char *buff)
{
   char *bp, *start;
   SDL_Cursor *c;
   int sendlen;
   
   bp = start = sdl_getbuff(sd, 4, buff);
   c = SDL_GetCursor();
   put32be(bp, (int) c);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;
}

int es_freeCursor(sdl_data *sd, int len,char *buff)
{
   char *bp;
   SDL_Cursor *c;
   
   bp = ARGS(buff);
   c = (SDL_Cursor *) get32be(bp);
   SDL_SetCursor(c);
   return 0;
}

int es_showCursor(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   Uint8 bool;

   bp = ARGS(buff);
   bool = get8(bp);
   bool = SDL_ShowCursor(bool);
   bp = start = sdl_getbuff(sd, 1, buff);
   put8(bp, bool);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         
}

int es_numJoysticks(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   int joys;

   bp = ARGS(buff);
   bp = start = sdl_getbuff(sd, 1, buff);
   joys = SDL_NumJoysticks();
   put8(bp, joys);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         
   
}
int es_joystick_name(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   int index;
   const char * name;
   bp = ARGS(buff);
   index = get8(bp);
   bp = start = sdl_getbuff(sd, 256, buff);
   name = SDL_JoystickName(index);
   index = 0;
   while(*name != '\0' && index++ < 256) {
      put8(bp, *name);
      name++;
   }
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         

}

int es_joystick_open(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   SDL_Joystick *joy;
   int index;
   
   bp = ARGS(buff);
   index = get8(bp);
   bp = start = sdl_getbuff(sd, 4, buff);
   if((joy = SDL_JoystickOpen(index)) != NULL) {
      putPointer(bp, joy);
   }
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;
}

int es_joystick_opened(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   int index, bool;
   
   bp = ARGS(buff);
   index = get8(bp);
   bp = start = sdl_getbuff(sd, 1, buff);
   bool = SDL_JoystickOpened(index);
   put8(bp, bool);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         

}

int es_joystick_index(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   SDL_Joystick *joy;
   int index;

   bp = ARGS(buff);
   joy = (SDL_Joystick *) get32be(bp);
   bp = start = sdl_getbuff(sd, 1, buff);
   index = SDL_JoystickIndex(joy);
   put8(bp,index);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         
}

int es_joystick_numAxes(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   SDL_Joystick *joy;
   int axes;

   bp = ARGS(buff);
   joy = (SDL_Joystick *) get32be(bp);
   bp = start = sdl_getbuff(sd, 1, buff);
   axes = SDL_JoystickNumAxes(joy);
   put8(bp,axes);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         
}

int es_joystick_numBalls(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   SDL_Joystick *joy;
   int balls;

   bp = ARGS(buff);
   joy = (SDL_Joystick *) get32be(bp);
   bp = start = sdl_getbuff(sd, 1, buff);
   balls = SDL_JoystickNumBalls(joy);
   put8(bp,balls);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         
}

int es_joystick_numHats(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   SDL_Joystick *joy;
   int hats;

   bp = ARGS(buff);
   joy = (SDL_Joystick *) get32be(bp);
   bp = start = sdl_getbuff(sd, 1, buff);
   hats = SDL_JoystickNumHats(joy);
   put8(bp,hats);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         
}

int es_joystick_numButtons(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   SDL_Joystick *joy;
   int buttons;

   bp = ARGS(buff);
   joy = (SDL_Joystick *) get32be(bp);
   bp = start = sdl_getbuff(sd, 1, buff);
   buttons = SDL_JoystickNumButtons(joy);
   put8(bp,buttons);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         
}

int es_joystick_update(sdl_data *sd, int len,char *buff)
{
   SDL_JoystickUpdate();
   return 0;         
}
int es_joystick_eventState(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   int state;
   
   bp = ARGS(buff);
   state = get32be(bp);
   bp = start = sdl_getbuff(sd, 4, buff);
   state = SDL_JoystickEventState(state);
   put32be(bp, state);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         

}
int es_joystick_getAxis(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   SDL_Joystick *joy;
   int state, axis;
   bp = ARGS(buff);
   joy = (SDL_Joystick *) get32be(bp);
   axis = get8(bp);
   bp = start = sdl_getbuff(sd, 4, buff);
   state = SDL_JoystickGetAxis(joy, axis);
   put32be(bp, state);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         

}
int es_joystick_getHat(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   SDL_Joystick *joy;
   int state;
   Uint8 hat;
   bp = ARGS(buff);
   joy = (SDL_Joystick *) get32be(bp);
   hat = get8(bp);
   bp = start = sdl_getbuff(sd, 1, buff);
   state = SDL_JoystickGetHat(joy, hat);
   put8(bp,state);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         
}
int es_joystick_getButton(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   SDL_Joystick *joy;
   int state;
   Uint8 button;
   bp = ARGS(buff);
   joy = (SDL_Joystick *) get32be(bp);
   button = get8(bp);
   bp = start = sdl_getbuff(sd, 1, buff);
   state = SDL_JoystickGetButton(joy, button);
   put8(bp,state);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         
}

int es_joystick_getBall(sdl_data *sd, int len,char *buff)
{
   int sendlen;
   char *bp, *start;
   SDL_Joystick *joy;
   int dx, dy;
   Uint8 ball;
   
   bp = ARGS(buff);
   joy = (SDL_Joystick *) get32be(bp);
   ball = get8(bp);
   bp = start = sdl_getbuff(sd, 8, buff);
   if(0 == SDL_JoystickGetBall(joy, ball, &dx, &dy)) {
      put32be(bp, dx);
      put32be(bp, dy);
   }
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();    
   return 0;         
}

int es_joystick_close(sdl_data *sd, int len,char *buff)
{
   char *bp;
   SDL_Joystick *joy;
   bp = ARGS(buff);
   joy = (SDL_Joystick *) get32be(bp);
   SDL_JoystickClose(joy);
   return 0;         
}

/* int es_GetWMInfo(int , char *); */
#define ESDL_FORWARD_EVENT 100

int forwardEvent(const SDL_Event *event)
{
   int sendlen;
   char *bp,  buff[32];   /* 32 should be enough according to encode function */
   bp = buff;
   put16be(bp, ESDL_FORWARD_EVENT);
   bp = encode_event(event, bp);
   sendlen = bp - buff;
   /* send NULL as sdl pointer shouldn't use threads with drivers anyway */
   if(sdl_send(NULL, sendlen, buff) != sendlen)
      error();
   /* Discard every event let them be handled in erlang */
   return 0; 
}

void setupEventHandler(void) 
{
   SDL_SetEventFilter(forwardEvent);
}

/* Internals */ 
char * encode_event(const SDL_Event * ev, char * bp)
{ 
   switch(ev->type) {
   case SDL_ACTIVEEVENT:	/* Application loses/gains visibility */
      put8(bp, SDL_ACTIVEEVENT);
      put8(bp, ev->active.gain);
      put8(bp, ev->active.state);	    
      break;
   case SDL_KEYDOWN:	/* Keys pressed */
   case SDL_KEYUP:	/* Keys released */ 
      put8(bp, SDL_KEYDOWN);   /* Using same for both */
      put8(bp, ev->key.which);
      put8(bp, ev->key.state);	    
      put8(bp, ev->key.keysym.scancode);	    
      put16be(bp, ev->key.keysym.sym);	    
      put16be(bp, ev->key.keysym.mod);	    
      put16be(bp, ev->key.keysym.unicode);	    
      break;
   case SDL_MOUSEMOTION:	/* Mouse moved */
      put8(bp, SDL_MOUSEMOTION);
      put8(bp, ev->motion.which);
      put8(bp, ev->motion.state);	    
      put16be(bp, ev->motion.x);
      put16be(bp, ev->motion.y);
      put16be(bp, ev->motion.xrel);
      put16be(bp, ev->motion.yrel);
      break;
   case SDL_MOUSEBUTTONDOWN:/* Mouse button pressed */
   case SDL_MOUSEBUTTONUP:/* Mouse button released */
      put8(bp, SDL_MOUSEBUTTONDOWN);
      put8(bp, ev->button.which);
      put8(bp, ev->button.button);
      put8(bp, ev->button.state);	    
      put16be(bp, ev->button.x);
      put16be(bp, ev->button.y);	
      break;
   case SDL_JOYAXISMOTION:/* Joystick axis motion */
      put8(bp, SDL_JOYAXISMOTION);
      put8(bp, ev->jaxis.which);
      put8(bp, ev->jaxis.axis);	    
      put16be(bp, ev->jaxis.value);
      break;
   case SDL_JOYBALLMOTION:/* Joystick trackball motion */
      put8(bp, SDL_JOYBALLMOTION);
      put8(bp, ev->jball.which);
      put8(bp, ev->jball.ball);	    
      put16be(bp, ev->jball.xrel);
      put16be(bp, ev->jball.yrel);
      break;
   case SDL_JOYHATMOTION:/* Joystick hat position change */
      put8(bp, SDL_JOYHATMOTION);
      put8(bp, ev->jhat.which);
      put8(bp, ev->jhat.hat);	    
      put16be(bp, ev->jhat.value);
      break;
   case SDL_JOYBUTTONDOWN:/* Joystick button pressed */
   case SDL_JOYBUTTONUP:	/* Joystick button released */
      put8(bp, SDL_JOYBUTTONDOWN);
      put8(bp, ev->jbutton.which);
      put8(bp, ev->jbutton.button);
      put8(bp, ev->jbutton.state);	    
      break;
   case SDL_QUIT:	/* User-requested quit */
      put8(bp, SDL_QUIT);
      break;
   case SDL_VIDEORESIZE:	
      put8(bp, SDL_VIDEORESIZE);
      put16be(bp, ev->resize.w);
      put16be(bp, ev->resize.h);
      break;
   case SDL_VIDEOEXPOSE:	
      put8(bp, SDL_VIDEOEXPOSE);
      break;

   case SDL_SYSWMEVENT:	/* System specific event */
	
      break;	
   default:   /* Forward compatible */
     fprintf(stderr, "ESDL received unsupported event type %x \n",
	     ev->type);
     put8(bp, SDL_NOEVENT);
     break;
   }
   return bp;
}




