
/*  
 *  Copyright (c) 2001 Dan Gudmundsson
 *  See the file "license.terms" for information on usage and redistribution
 *  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 *     $Id: esdl_glu.c,v 1.27 2003/03/14 09:12:56 dgud Exp $
 *
 *  
 *  GLU wrapper 
 *
 */

#include <stdio.h>  /* malloc */
#include <stdlib.h>  /* malloc */
#include <string.h>  /* malloc */
#include <math.h>  /* malloc */
#include "esdl.h"
#ifdef _OSX_COCOA
#include <OpenGL/glu.h>
#else
#include <GL/glu.h>
#endif
#define ASIZE(a) (sizeof(a)/sizeof(a[0]))

#define GetNativePtr(res, T, s) \
do { res = T *(void **)(s); s += sizeof(void *); } while (0)

typedef struct _tessdata3 * eglu_tessdata_ptr;

typedef struct _tessdata3 {
		 eglu_tessdata_ptr next;
		 GLdouble data[3];
		} eglu_tessdata; 

typedef struct _tessobj {   
		 GLUtesselator *tess;
		 eglu_tessdata *data;
		 GLdouble* freep;
		 GLdouble def_heap[64];
		} eglu_tessobj;

#ifndef CALLBACK
# define CALLBACK
#endif

#define ESDL_TESSCB_NONE        0       /* No callback                         */
#define ESDL_TESSCB_GLBEGIN     1       /* calls gl:begin(Type)                */
#define ESDL_TESSCB_GLEND       2       /* calls gl:end()                      */
#define ESDL_TESSCB_GLVERTEX    3       /* calls gl:3dv(vertex)                */
#define ESDL_TESSCB_ERROR_PRINT 4       /* Prints error on stderr.             */
#define ESDL_TESSCB_COMBINE     5       /* Adds vertex                   */
#define ESDL_TESSCB_GLEDGEFLAG  6       /* calls gl:edgeFlag(Flag)       */

#define ESDL_TESSCB_UDATA_VERTEX  7 /* calls various gl functions
				     * depending on user data
				     */

#define ESDL_TESS_VTXDATA_MATERIAL     1
#define ESDL_TESS_VTXDATA_TEXCOORD2    2
#define ESDL_TESS_VTXDATA_NORMAL       4
#define ESDL_TESS_VTXDATA_COLOR        8

void CALLBACK errorCallback(GLenum errorCode)
{
   const GLubyte *err;
   err = gluErrorString(errorCode);
   fprintf(stderr, "Tesselation error: %d: %s\r\n", errorCode, err);
}

void CALLBACK esdl_combine(GLdouble coords[3], 
			   void *vertex_data[4],
			   GLfloat w[4], 
			   void **outData,
			   void *polygon_data
			  )
{
   int size = 0;
   int max = 0;
   eglu_tessobj *eobj = polygon_data;
   eglu_tessdata* mycoords = NULL; 
   unsigned char* combined  = NULL; 
   unsigned char* datap[4] = {NULL, NULL, NULL, NULL};
   unsigned flags;

   flags = ((unsigned char *)vertex_data[0])[-1];
   for (max = 0; max < 4 && vertex_data[max] != NULL; max++) {
      if (((unsigned char *)vertex_data[max])[-1] != flags) {
	 flags = 0;
      }
      datap[max] = (unsigned char *) (((GLdouble *)vertex_data[max])+3);
   }

   if (flags & ESDL_TESS_VTXDATA_MATERIAL) {
      size += 5*4;
   }
   if (flags & ESDL_TESS_VTXDATA_TEXCOORD2) {
      size += 2*4;
   }
   if (flags & ESDL_TESS_VTXDATA_NORMAL) {
      size += 3*4;
   }
   if (flags & ESDL_TESS_VTXDATA_COLOR) {
      size += 2*4;
   }

   mycoords = (eglu_tessdata*) malloc(sizeof(GLdouble) + 
				      sizeof(eglu_tessdata) + size);
   combined = (unsigned char *) (mycoords->data+4);

   mycoords->next = eobj->data;
   eobj->data = mycoords;
   
   mycoords->data[1] = coords[0];
   mycoords->data[2] = coords[1];
   mycoords->data[3] = coords[2];
   
   *outData = mycoords->data + 1;
   flags = 0;
   ((char *) (mycoords->data+1))[-1] = flags;
}

void CALLBACK
esdl_udata_vertex(GLdouble* coords)
{
   GLfloat* datap = (GLfloat *) (coords+3);
   unsigned flags = ((unsigned char *) coords)[-1];

   /*   fprintf(stderr, "Flags: %d\r\n", flags); */
   if (flags & ESDL_TESS_VTXDATA_MATERIAL) {
      GLenum face = ((unsigned short *) datap)[0];
      GLenum pname = ((unsigned short *) datap)[1];
      datap++;
      glMaterialfv(face, pname, datap);
      datap += 4;
   }
   if (flags & ESDL_TESS_VTXDATA_TEXCOORD2) {
      glTexCoord2fv(datap);
      datap += 2;
   }
   if (flags & ESDL_TESS_VTXDATA_NORMAL) {
      glNormal3fv(datap);
      datap += 3;
   }
   if (flags & ESDL_TESS_VTXDATA_COLOR) {
      glColor4fv(datap);
      datap += 4;
   }
   glVertex3dv(coords);
}

void eglu_newTess (sdl_data *sd, int len, char * buff) 
{
   char *bp, *start;
   int sendlen; 
   GLUtesselator* tobj;
   eglu_tessobj * eobj;
    
   tobj = gluNewTess();
   eobj = (eglu_tessobj *) malloc(sizeof(eglu_tessobj));
   eobj->tess = tobj;
   eobj->data = NULL;
   eobj->freep = eobj->def_heap;

/*    fprintf(stderr, "New tess: %d -> %d \r\n", (int) eobj, (int) tobj);  */

   /*
    * Send back result.
    */
   bp = start = sdl_getbuff(sd, 4, buff);    
   /* putPointer(bp, eobj); */
   * (eglu_tessobj **) bp = eobj;
   bp += sizeof(eglu_tessobj *);

   sendlen = bp - start;
   sdl_send(sd, sendlen, buff);
}

void eglu_deleteTess (sdl_data *sd, int len, char * buff) 
{
   char *bp;
   eglu_tessobj *eobj;
   bp = buff;
   GetNativePtr(eobj, (eglu_tessobj *), bp);
   gluDeleteTess(eobj->tess);
   free(eobj);
/*    fprintf(stderr, "Deleting tess: %d\r\n", (int) eobj); */
}

void eglu_tessBeginPolygon (sdl_data *sd, int len, char * buff) 
{
   char *bp;
   eglu_tessobj *eobj;

   /* void *polygon_data; */
   bp = buff;
   GetNativePtr(eobj, (eglu_tessobj *), bp);
/*    fprintf(stderr, "tessBeginPolygon: %d\r\n", (int) eobj); */
   gluTessBeginPolygon(eobj->tess, (void *) eobj);
}

void eglu_tessVertex(sdl_data *sd, int len, char* bp) 
{
   eglu_tessobj *eobj;
   eglu_tessdata* coords; 
   size_t size;
   size_t ndoubles;
   size_t nbytes;
   char* extrap;

   GetNativePtr(eobj, (eglu_tessobj *), bp);
   size = len - sizeof(void *);
   nbytes = sizeof(GLdouble) + sizeof(eglu_tessdata) + size;
   ndoubles = (nbytes-1)/sizeof(GLdouble) + 1;
   if (ndoubles <= eobj->def_heap+ASIZE(eobj->def_heap)-eobj->freep) {
      coords = (eglu_tessdata*) eobj->freep;
      eobj->freep += ndoubles;
   } else {
      coords = (eglu_tessdata*) malloc(nbytes);
      coords->next = eobj->data;
      eobj->data = coords;
   }
   memcpy(coords->data+1, bp, size);
   extrap = (char *) (coords->data+1);
   if (size > 3*sizeof(GLdouble)) {
      extrap[-1] = bp[size-1];
   } else {
      extrap[-1] = 0;
   }
/*     fprintf(stderr, "tessVertex: %d %g %g %g\r\n", */
/* 	    (int) eobj, coords->data[0], coords->data[1], coords->data[2]); */
    gluTessVertex(eobj->tess, coords->data+1, coords->data+1);
}

void eglu_tessEndPolygon (sdl_data *sd, int len, char * buff) 
{
   char *bp;
   eglu_tessobj *eobj;
   eglu_tessdata *remove, *temp;

   bp = buff;
   GetNativePtr(eobj, (eglu_tessobj *), bp);
   gluTessEndPolygon(eobj->tess);
   remove = eobj->data;
   while (remove != NULL) {
      temp = remove->next;
      free(remove);
      remove = temp;
   }
   eobj->data = NULL;
   eobj->freep = eobj->def_heap;

/*    fprintf(stderr, "End Polygon: %d\r\n", (int) eobj); */

}

void eglu_tessCallback(sdl_data *sd, int len, char * buff) 
{
   char *bp;
   eglu_tessobj *eobj;
   GLenum *which;
   int *cbId;
   GLvoid (CALLBACK *cbfn)();
   bp = buff;
   GetNativePtr(eobj, (eglu_tessobj *), bp);
   which = (GLenum *) bp; bp += sizeof(GLenum); 
   cbId = (GLint *) bp; bp += sizeof(GLint); 

   switch(*which) {
   case GLU_TESS_COMBINE:
      *which = GLU_TESS_COMBINE_DATA;
      break;      
   default:
      break;
   };
      
   switch(*cbId) {
   case ESDL_TESSCB_NONE:
      cbfn = NULL;
      break;
   case ESDL_TESSCB_GLBEGIN:
      cbfn = (GLvoid (*)()) glBegin;
      break;
   case ESDL_TESSCB_GLEND:
      cbfn = (GLvoid (*)()) glEnd;
      break;
   case ESDL_TESSCB_GLVERTEX:
      cbfn = (GLvoid (*)()) glVertex3dv;
      break;
   case ESDL_TESSCB_GLEDGEFLAG:
      cbfn = (GLvoid (*)()) glEdgeFlag;
      break;
   case ESDL_TESSCB_ERROR_PRINT:
      cbfn = (GLvoid (*)()) errorCallback;
      break;
   case ESDL_TESSCB_COMBINE:
      cbfn = (GLvoid (*)()) esdl_combine;
      break;
   case ESDL_TESSCB_UDATA_VERTEX:
      cbfn = (GLvoid (*)()) esdl_udata_vertex;
      break;

   default:
/*        fprintf(stderr, "Bad Callback: %d %d\r\n", (int) eobj, *cbId); */
       cbfn = NULL;
       break;
   };
   gluTessCallback(eobj->tess, *which, cbfn);
/*    fprintf(stderr, "Tess Callback: %d %d\r\n", (int) eobj, *cbId); */
}
void eglu_beginCurve(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUnurbs * nurb;
   bp = egl_buff;
   GetNativePtr(nurb, (GLUnurbs *),bp);
   gluBeginCurve(nurb);

}

void eglu_beginPolygon(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   eglu_tessBeginPolygon(egl_sd, egl_len, egl_buff);
}

void eglu_beginSurface(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUnurbs * nurb;
   bp = egl_buff;
   GetNativePtr(nurb, (GLUnurbs *),bp);
   gluBeginSurface(nurb);

}

void eglu_beginTrim(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUnurbs * nurb;
   bp = egl_buff;
   GetNativePtr(nurb, (GLUnurbs *),bp);
   gluBeginTrim(nurb);

}

void eglu_build1DMipmaps(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   char * egl_start; 
   int egl_sendlen; 
   GLint egl_res; 
   GLenum * target;
   GLint * internalFormat;
   GLsizei * width;
   GLenum * format;
   GLenum * type;
   int * dataMemT = 0;
   int * dataLen = 0;
   void * data = NULL;
   bp = egl_buff;
   target = (GLenum *) bp; bp += sizeof(GLenum); 
   internalFormat = (GLint *) bp; bp += sizeof(GLint); 
   width = (GLsizei *) bp; bp += sizeof(GLsizei); 
   format = (GLenum *) bp; bp += sizeof(GLenum); 
   type = (GLenum *) bp; bp += sizeof(GLenum); 
   dataMemT = (int *)bp; bp += sizeof(int);
   if(*dataMemT == 1) {
      data = (void *) getPointer(bp);
      data = usePointer((oglmem *)data);} else {
	 dataLen = (int *)bp; bp += sizeof(int); 
	 data = (void *) bp; bp += *dataLen;};
   egl_res =  gluBuild1DMipmaps(*target, *internalFormat, *width, *format, *type, data);
   bp = egl_start = sdl_getbuff(egl_sd, sizeof(GLint) , egl_buff);
   * (GLint *) bp = egl_res;
   bp += sizeof(GLint);
   egl_sendlen = bp - egl_start;
   sdl_send(egl_sd, egl_sendlen, egl_buff);

}

void eglu_build2DMipmaps(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   char * egl_start; 
   int egl_sendlen; 
   GLint egl_res; 
   GLenum * target;
   GLint * internalFormat;
   GLsizei * width;
   GLsizei * height;
   GLenum * format;
   GLenum * type;
   int * dataMemT = 0;
   int * dataLen = 0;
   void * data = NULL;
   bp = egl_buff;
   target = (GLenum *) bp; bp += sizeof(GLenum); 
   internalFormat = (GLint *) bp; bp += sizeof(GLint); 
   width = (GLsizei *) bp; bp += sizeof(GLsizei); 
   height = (GLsizei *) bp; bp += sizeof(GLsizei); 
   format = (GLenum *) bp; bp += sizeof(GLenum); 
   type = (GLenum *) bp; bp += sizeof(GLenum); 
   dataMemT = (int *)bp; bp += sizeof(int);
   if(*dataMemT == 1) {
      data = (void *) getPointer(bp);
      data = usePointer((oglmem *)data);} else {
	 dataLen = (int *)bp; bp += sizeof(int); 
	 data = (void *) bp; bp += *dataLen;};
   egl_res =  gluBuild2DMipmaps(*target, *internalFormat, *width, 
				*height, *format, *type, data);
   bp = egl_start = sdl_getbuff(egl_sd, sizeof(GLint) , egl_buff);
   * (GLint *) bp = egl_res;
   bp += sizeof(GLint);
   egl_sendlen = bp - egl_start;
   sdl_send(egl_sd, egl_sendlen, egl_buff);

}

/* void eglu_build3DMipmaps(sdl_data *egl_sd, int egl_len, char *egl_buff)  */
/* { */
/*    char * bp;  */
/*    char * egl_start;  */
/*    int egl_sendlen;  */
/*    GLint egl_res;  */
/*    GLenum * target; */
/*    GLint * internalFormat; */
/*    GLsizei * width; */
/*    GLsizei * height; */
/*    GLsizei * depth; */
/*    GLenum * format; */
/*    GLenum * type; */
/*    int * dataMemT = 0; */
/*    int * dataLen = 0; */
/*    void * data = NULL; */
/*    bp = egl_buff; */
/*    target = (GLenum *) bp; bp += sizeof(GLenum);  */
/*    internalFormat = (GLint *) bp; bp += sizeof(GLint);  */
/*    width = (GLsizei *) bp; bp += sizeof(GLsizei);  */
/*    height = (GLsizei *) bp; bp += sizeof(GLsizei);  */
/*    depth = (GLsizei *) bp; bp += sizeof(GLsizei);  */
/*    format = (GLenum *) bp; bp += sizeof(GLenum);  */
/*    type = (GLenum *) bp; bp += sizeof(GLenum);  */
/*    dataMemT = (int *)bp; bp += sizeof(int); */
/*    if(*dataMemT == 1) { */
/*       data = (void *) getPointer(bp); */
/*       data = usePointer((oglmem *)data);} else { */
/* 	 dataLen = (int *)bp; bp += sizeof(int);  */
/* 	 data = (void *) bp; bp += *dataLen;}; */
/*    egl_res =  gluBuild3DMipmaps(*target, *internalFormat, *width, *height, *depth, *format, *type, data); */
/*    bp = egl_start = sdl_getbuff(egl_sd, sizeof(GLint) , egl_buff); */
/*    * (GLint *) bp = egl_res; */
/*    bp += sizeof(GLint); */
/*    egl_sendlen = bp - egl_start; */
/*    sdl_send(egl_sd, egl_sendlen, egl_buff); */

/* } */

void eglu_cylinder(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUquadric * quad;
   GLdouble base;
   GLdouble top;
   GLdouble height;
   GLint * slices;
   GLint * stacks;
   bp = egl_buff;
   GetNativePtr(quad, (GLUquadric *),bp);
   memcpy(&base, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(&top, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(&height, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   slices = (GLint *) bp; bp += sizeof(GLint); 
   stacks = (GLint *) bp; bp += sizeof(GLint); 
   gluCylinder(quad, base, top, height, *slices, *stacks);

}

void eglu_deleteNurbsRenderer(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUnurbs * nurb;
   bp = egl_buff;
   GetNativePtr(nurb, (GLUnurbs *),bp);
   gluDeleteNurbsRenderer(nurb);

}

void eglu_deleteQuadric(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUquadric * quad;
   bp = egl_buff;
   GetNativePtr(quad, (GLUquadric *),bp);
   gluDeleteQuadric(quad);

}

void eglu_disk(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUquadric * quad;
   GLdouble inner;
   GLdouble outer;
   GLint * slices;
   GLint * loops;
   bp = egl_buff;
   GetNativePtr(quad, (GLUquadric *),bp);
   memcpy(&inner, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(&outer, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   slices = (GLint *) bp; bp += sizeof(GLint); 
   loops = (GLint *) bp; bp += sizeof(GLint); 
   gluDisk(quad, inner, outer, *slices, *loops);

}

void eglu_endCurve(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUnurbs * nurb;
   bp = egl_buff;
   GetNativePtr(nurb, (GLUnurbs *),bp);
   gluEndCurve(nurb);

}

void eglu_endPolygon(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   eglu_tessEndPolygon(egl_sd,egl_len,egl_buff);
}

void eglu_endSurface(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUnurbs * nurb;
   bp = egl_buff;
   GetNativePtr(nurb, (GLUnurbs *),bp);
   gluEndSurface(nurb);

}

void eglu_endTrim(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUnurbs * nurb;
   bp = egl_buff;
   GetNativePtr(nurb, (GLUnurbs *),bp);
   gluEndTrim(nurb);

}

void eglu_errorString(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   char * egl_start; 
   int egl_sendlen; 
   const GLubyte *egl_res; 
   GLenum * error;
   bp = egl_buff;
   error = (GLenum *) bp; bp += sizeof(GLenum); 
   egl_res = gluErrorString(*error);
   bp = egl_start = sdl_getbuff(egl_sd, strlen(egl_res) , egl_buff);
   strcpy((GLubyte *)bp, egl_res);
   bp += strlen(egl_res);
   egl_sendlen = bp - egl_start;
   sdl_send(egl_sd, egl_sendlen, egl_buff);

}

void eglu_getNurbsProperty(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   char * egl_start; 
   int egl_sendlen; 
   GLUnurbs * nurb;
   GLenum * property;
   GLfloat data[1]; 
   bp = egl_buff;
   GetNativePtr(nurb, (GLUnurbs *),bp);
   property = (GLenum *) bp; bp += sizeof(GLenum); 
   gluGetNurbsProperty(nurb, *property, data);
   bp = egl_start = sdl_getbuff(egl_sd, sizeof(GLfloat) *1, egl_buff);
   * (GLfloat *)bp = data[0]; bp += sizeof(GLfloat);
   egl_sendlen = bp - egl_start;
   sdl_send(egl_sd, egl_sendlen, egl_buff);

}

void eglu_getString(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   char * egl_start; 
   int egl_sendlen; 
   const GLubyte *egl_res; 
   GLenum * name;
   bp = egl_buff;
   name = (GLenum *) bp; bp += sizeof(GLenum); 
   egl_res = gluGetString(*name);
   bp = egl_start = sdl_getbuff(egl_sd, strlen(egl_res) , egl_buff);
   strcpy((GLubyte *)bp, egl_res);
   bp += strlen(egl_res);
   egl_sendlen = bp - egl_start;
   sdl_send(egl_sd, egl_sendlen, egl_buff);

}

void eglu_getTessProperty(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   char * egl_start; 
   int egl_sendlen; 
   eglu_tessobj * tess;
   GLenum * which;
   GLdouble data[1]; 
   bp = egl_buff;
   GetNativePtr(tess, (eglu_tessobj *),bp);
   which = (GLenum *) bp; bp += sizeof(GLenum); 
   gluGetTessProperty(tess->tess, *which, data);
   bp = egl_start = sdl_getbuff(egl_sd, sizeof(GLdouble) *1, egl_buff);
   * (GLdouble *)bp = data[0]; bp += sizeof(GLdouble);
   egl_sendlen = bp - egl_start;
   sdl_send(egl_sd, egl_sendlen, egl_buff);

}

void eglu_loadSamplingMatrices(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUnurbs * nurb;
   GLfloat * model;
   GLfloat * perspective;
   GLint * view;
   bp = egl_buff;
   GetNativePtr(nurb, (GLUnurbs *),bp);
   model = (GLfloat *) bp; bp += sizeof(GLfloat)*16; 
   perspective = (GLfloat *) bp; bp += sizeof(GLfloat)*16; 
   view = (GLint *) bp; bp += sizeof(GLint)*16; 
   gluLoadSamplingMatrices(nurb, model, perspective, view);

}

void eglu_lookAt(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLdouble data[9];
   bp = egl_buff;
   memcpy(data, bp, sizeof(GLdouble)*9); bp += sizeof(GLdouble)*9; 
   gluLookAt(data[0],data[1],data[2],
	     data[3],data[4],data[5],
	     data[6],data[7],data[8]);

}

void eglu_newNurbsRenderer(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   char * egl_start; 
   int egl_sendlen; 
   GLUnurbs *egl_res; 
   bp = egl_buff;
   egl_res =  gluNewNurbsRenderer();
   bp = egl_start = sdl_getbuff(egl_sd, sizeof(GLUnurbs*) , egl_buff);
   * (GLUnurbs **) bp = egl_res;
   bp += sizeof(GLUnurbs *);
   egl_sendlen = bp - egl_start;
   sdl_send(egl_sd, egl_sendlen, egl_buff);

}

void eglu_newQuadric(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   char * egl_start; 
   int egl_sendlen; 
   GLUquadric *egl_res; 
   bp = egl_buff;
   egl_res =  gluNewQuadric();
   bp = egl_start = sdl_getbuff(egl_sd, sizeof(GLUquadric*) , egl_buff);
   * (GLUquadric **) bp = egl_res;
   bp += sizeof(GLUquadric *);
   egl_sendlen = bp - egl_start;
   sdl_send(egl_sd, egl_sendlen, egl_buff);

}

void eglu_nextContour(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   eglu_tessobj * tess;
   GLenum * type;
   bp = egl_buff;
   GetNativePtr(tess, (eglu_tessobj *),bp);
   type = (GLenum *) bp; bp += sizeof(GLenum); 
   gluNextContour(tess->tess, *type);

}

void eglu_nurbsCurve(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUnurbs * nurb;
   GLint * knotCount;
   GLfloat * knots = NULL; int * knotsLen;
   GLint * stride;
   int * controlMemT = 0;
   int * controlLen = 0;
   GLfloat * control = NULL;
   GLint * order;
   GLenum * type;
   bp = egl_buff;
   GetNativePtr(nurb, (GLUnurbs *),bp);
   knotCount = (GLint *) bp; bp += sizeof(GLint); 
   knotsLen = knotCount;
   knots = (GLfloat *) bp; bp += sizeof(GLfloat)*(*knotsLen); 
   stride = (GLint *) bp; bp += sizeof(GLint); 
   controlMemT = (int *)bp; bp += sizeof(int);
   if(*controlMemT == 1) {
      control = (GLfloat *) getPointer(bp);
      control = usePointer((oglmem *)control);
   } else {
      controlLen = (int *)bp; bp += sizeof(int); 
      control = (GLfloat *) bp; bp += *controlLen;
   };
   order = (GLint *) bp; bp += sizeof(GLint); 
   type = (GLenum *) bp; bp += sizeof(GLenum); 
   gluNurbsCurve(nurb, *knotCount, knots, *stride, control, *order, *type);

}

void eglu_nurbsProperty(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUnurbs * nurb;
   GLenum * property;
   GLfloat * value;
   bp = egl_buff;
   GetNativePtr(nurb, (GLUnurbs *),bp);
   property = (GLenum *) bp; bp += sizeof(GLenum); 
   value = (GLfloat *) bp; bp += sizeof(GLfloat); 
   gluNurbsProperty(nurb, *property, *value);

}

void eglu_nurbsSurface(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUnurbs * nurb;
   GLint * sKnotCount;
   GLfloat * sKnots = NULL; int * sKnotsLen;
   GLint * tKnotCount;
   GLfloat * tKnots = NULL; int * tKnotsLen;
   GLint * sStride;
   GLint * tStride;
   int * controlMemT = 0;
   int * controlLen = 0;
   GLfloat * control = NULL;
   GLint * sOrder;
   GLint * tOrder;
   GLenum * type;
   bp = egl_buff;
   GetNativePtr(nurb, (GLUnurbs *),bp);
   sKnotCount = (GLint *) bp; bp += sizeof(GLint); 
   sKnotsLen = sKnotCount;
   sKnots = (GLfloat *) bp; bp += sizeof(GLfloat)*(*sKnotsLen); 
   tKnotCount = (GLint *) bp; bp += sizeof(GLint); 
   tKnotsLen = tKnotCount;
   tKnots = (GLfloat *) bp; bp += sizeof(GLfloat)*(*tKnotsLen); 
   sStride = (GLint *) bp; bp += sizeof(GLint); 
   tStride = (GLint *) bp; bp += sizeof(GLint); 
   controlMemT = (int *)bp; bp += sizeof(int);
   if(*controlMemT == 1) {
      control = (GLfloat *) getPointer(bp);
      control = usePointer((oglmem *)control);} else {
	 controlLen = (int *)bp; bp += sizeof(int); 
	 control = (GLfloat *) bp; bp += *controlLen;};
   sOrder = (GLint *) bp; bp += sizeof(GLint); 
   tOrder = (GLint *) bp; bp += sizeof(GLint); 
   type = (GLenum *) bp; bp += sizeof(GLenum); 
   gluNurbsSurface(nurb, *sKnotCount, sKnots, *tKnotCount, tKnots, *sStride, *tStride, control, *sOrder, *tOrder, *type);

}

void eglu_ortho2D(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLdouble left;
   GLdouble right;
   GLdouble bottom;
   GLdouble top;
   bp = egl_buff;
   memcpy(&left, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(&right, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(&bottom, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(&top, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   gluOrtho2D(left, right, bottom, top);

}

void eglu_partialDisk(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUquadric * quad;
   GLdouble inner;
   GLdouble outer;
   GLint * slices;
   GLint * loops;
   GLdouble start;
   GLdouble sweep;
   bp = egl_buff;
   GetNativePtr(quad, (GLUquadric *),bp);
   memcpy(&inner, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(&outer, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   slices = (GLint *) bp; bp += sizeof(GLint); 
   loops = (GLint *) bp; bp += sizeof(GLint); 
   memcpy(&start, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(&sweep, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   gluPartialDisk(quad, inner, outer, *slices, *loops, start, sweep);

}

void eglu_perspective(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLdouble fovy;
   GLdouble aspect;
   GLdouble zNear;
   GLdouble zFar;
   bp = egl_buff;
   memcpy(&fovy, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(&aspect, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(&zNear, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(&zFar, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   gluPerspective(fovy, aspect, zNear, zFar);

}

void eglu_pickMatrix(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLdouble x;
   GLdouble y;
   GLdouble delX;
   GLdouble delY;
   GLint * viewport;
   bp = egl_buff;
   memcpy(&x, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(&y, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(&delX, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(&delY, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   viewport = (GLint *) bp; bp += sizeof(GLint)*4; 
   gluPickMatrix(x, y, delX, delY, viewport);

}

void eglu_project(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   char * egl_start; 
   int egl_sendlen; 
   GLint egl_res; 
   GLdouble objX;
   GLdouble objY;
   GLdouble objZ;
   GLdouble model[16];
   GLdouble proj[16];
   GLint * view;
   GLdouble winX[1]; 
   GLdouble winY[1]; 
   GLdouble winZ[1]; 
   bp = egl_buff;
   memcpy(&objX, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(&objY, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(&objZ, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(model,bp,sizeof(GLdouble)*16); bp += sizeof(GLdouble)*16; 
   memcpy(proj,bp,sizeof(GLdouble)*16); bp += sizeof(GLdouble)*16; 
   view = (GLint *) bp; bp += sizeof(GLint)*16; 
   egl_res =  gluProject(objX, objY, objZ, model, proj, view, winX, winY, winZ);
   bp = egl_start = sdl_getbuff(egl_sd, sizeof(GLint) + sizeof(GLdouble) *1+ sizeof(GLdouble) *1+ sizeof(GLdouble) *1, egl_buff);
   * (GLint *) bp = egl_res;
   bp += sizeof(GLint);
   * (GLdouble *)bp = winX[0]; bp += sizeof(GLdouble);
   * (GLdouble *)bp = winY[0]; bp += sizeof(GLdouble);
   * (GLdouble *)bp = winZ[0]; bp += sizeof(GLdouble);
   egl_sendlen = bp - egl_start;
   sdl_send(egl_sd, egl_sendlen, egl_buff);

}

void eglu_pwlCurve(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUnurbs * nurb;
   GLint * count;
   int * dataMemT = 0;
   int * dataLen = 0;
   GLfloat * data = NULL;
   GLint * stride;
   GLenum * type;
   bp = egl_buff;
   GetNativePtr(nurb, (GLUnurbs *),bp);
   count = (GLint *) bp; bp += sizeof(GLint); 
   dataMemT = (int *)bp; bp += sizeof(int);
   if(*dataMemT == 1) {
      data = (GLfloat *) getPointer(bp);
      data = usePointer((oglmem *)data);} else {
	 dataLen = (int *)bp; bp += sizeof(int); 
	 data = (GLfloat *) bp; bp += *dataLen;};
   stride = (GLint *) bp; bp += sizeof(GLint); 
   type = (GLenum *) bp; bp += sizeof(GLenum); 
   gluPwlCurve(nurb, *count, data, *stride, *type);

}

void eglu_quadricDrawStyle(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUquadric * quad;
   GLenum * draw;
   bp = egl_buff;
   GetNativePtr(quad, (GLUquadric *),bp);
   draw = (GLenum *) bp; bp += sizeof(GLenum); 
   gluQuadricDrawStyle(quad, *draw);

}

void eglu_quadricNormals(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUquadric * quad;
   GLenum * normal;
   bp = egl_buff;
   GetNativePtr(quad, (GLUquadric *),bp);
   normal = (GLenum *) bp; bp += sizeof(GLenum); 
   gluQuadricNormals(quad, *normal);

}

void eglu_quadricOrientation(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUquadric * quad;
   GLenum * orientation;
   bp = egl_buff;
   GetNativePtr(quad, (GLUquadric *),bp);
   orientation = (GLenum *) bp; bp += sizeof(GLenum); 
   gluQuadricOrientation(quad, *orientation);

}

void eglu_quadricTexture(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUquadric * quad;
   GLboolean * texture;
   bp = egl_buff;
   GetNativePtr(quad, (GLUquadric *),bp);
   texture = (GLboolean *) bp; bp += sizeof(GLboolean); 
   gluQuadricTexture(quad, *texture);

}

void eglu_scaleImage(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   char * egl_start; 
   int egl_sendlen; 
   GLint egl_res; 
   GLenum * format;
   GLsizei * wIn;
   GLsizei * hIn;
   GLenum * typeIn;
   int * dataInMemT = 0;
   int * dataInLen = 0;
   void * dataIn = NULL;
   GLsizei * wOut;
   GLsizei * hOut;
   GLenum * typeOut;
   oglmem * dataOut = NULL;
   bp = egl_buff;
   format = (GLenum *) bp; bp += sizeof(GLenum); 
   wIn = (GLsizei *) bp; bp += sizeof(GLsizei); 
   hIn = (GLsizei *) bp; bp += sizeof(GLsizei); 
   typeIn = (GLenum *) bp; bp += sizeof(GLenum); 
   dataInMemT = (int *)bp; bp += sizeof(int);
   if(*dataInMemT == 1) {
      dataIn = (void *) getPointer(bp);
      dataIn = usePointer((oglmem *)dataIn);} else {
	 dataInLen = (int *)bp; bp += sizeof(int); 
	 dataIn = (void *) bp; bp += *dataInLen;};
   wOut = (GLsizei *) bp; bp += sizeof(GLsizei); 
   hOut = (GLsizei *) bp; bp += sizeof(GLsizei); 
   typeOut = (GLenum *) bp; bp += sizeof(GLenum); 
   dataOut = getPointer(bp); 
   egl_res =  gluScaleImage(*format, *wIn, *hIn, *typeIn, dataIn, *wOut, *hOut, *typeOut, usePointer(dataOut));
   bp = egl_start = sdl_getbuff(egl_sd, sizeof(GLint) , egl_buff);
   * (GLint *) bp = egl_res;
   bp += sizeof(GLint);
   egl_sendlen = bp - egl_start;
   sdl_send(egl_sd, egl_sendlen, egl_buff);

}

void eglu_sphere(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   GLUquadric * quad;
   GLdouble radius;
   GLint * slices;
   GLint * stacks;
   bp = egl_buff;
   GetNativePtr(quad, (GLUquadric *),bp);
   memcpy(&radius, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   slices = (GLint *) bp; bp += sizeof(GLint); 
   stacks = (GLint *) bp; bp += sizeof(GLint); 
   gluSphere(quad, radius, *slices, *stacks);

}

void eglu_tessBeginContour(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   eglu_tessobj * tess;
   bp = egl_buff;
   GetNativePtr(tess, (eglu_tessobj *),bp);
   gluTessBeginContour(tess->tess);
/*    fprintf(stderr, "Begin Contour: %d\r\n", (int) tess); */

}

void eglu_tessEndContour(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   eglu_tessobj * tess;
   bp = egl_buff;
   GetNativePtr(tess, (eglu_tessobj *),bp);
   gluTessEndContour(tess->tess);
/*    fprintf(stderr, "Begin Contour: %d\r\n", (int) tess); */
}


void eglu_tessNormal(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   eglu_tessobj * tess;
   GLdouble values[3];
   bp = egl_buff;
   GetNativePtr(tess, (eglu_tessobj *),bp);
   memcpy(values, bp, sizeof(GLdouble)*3); bp += sizeof(GLdouble)*3; 
   gluTessNormal(tess->tess, values[0], values[1], values[2]);

}

void eglu_tessProperty(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   eglu_tessobj * tess;
   GLenum * which;
   GLdouble data;
   bp = egl_buff;
   GetNativePtr(tess, (eglu_tessobj *),bp);
   which = (GLenum *) bp; bp += sizeof(GLenum); 
   memcpy(&data, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   gluTessProperty(tess->tess, *which, data);

}

void eglu_unProject(sdl_data *egl_sd, int egl_len, char *egl_buff) 
{
   char * bp; 
   char * egl_start; 
   int egl_sendlen; 
   GLint egl_res; 
   GLdouble winX;
   GLdouble winY;
   GLdouble winZ;
   GLdouble model[16];
   GLdouble proj[16];
   GLint * view;
   GLdouble objX[1]; 
   GLdouble objY[1]; 
   GLdouble objZ[1]; 
   bp = egl_buff;
   memcpy(&winX, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(&winY, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(&winZ, bp, sizeof(GLdouble)); bp += sizeof(GLdouble); 
   memcpy(model,bp,sizeof(GLdouble)*16); bp += sizeof(GLdouble)*16; 
   memcpy(proj,bp,sizeof(GLdouble)*16); bp += sizeof(GLdouble)*16; 
   view = (GLint *) bp; bp += sizeof(GLint)*16; 
   egl_res =  gluUnProject(winX, winY, winZ, model, proj, view, objX, objY, objZ);
   bp = egl_start = sdl_getbuff(egl_sd, sizeof(GLint) + sizeof(GLdouble) *1+ sizeof(GLdouble) *1+ sizeof(GLdouble) *1, egl_buff);
   * (GLint *) bp = egl_res;
   bp += sizeof(GLint);
   * (GLdouble *)bp = objX[0]; bp += sizeof(GLdouble);
   * (GLdouble *)bp = objY[0]; bp += sizeof(GLdouble);
   * (GLdouble *)bp = objZ[0]; bp += sizeof(GLdouble);
   egl_sendlen = bp - egl_start;
   sdl_send(egl_sd, egl_sendlen, egl_buff);

}

