/*
 *  Copyright (c) 2001 Dan Gudmundsson
 *  See the file "license.terms" for information on usage and redistribution
 *  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 *     $Id: esdl_util.c,v 1.2 2001/11/22 15:53:36 dgud Exp $
 */
/* 
  Some useful extra functions   
 */

#ifdef _WIN32
#include <windows.h>
#endif

#include <stdlib.h>  /* malloc */
#include "esdl.h"

int mygl_alloc(sdl_data *sd, int len, char *buff)
{
   char *bp, *start;
   int sendlen, size;
   GLenum type;
   GLvoid *mem;
   oglmem *ref;
   bp = ARGS(buff);
   size = get32be(bp);
   type = get16be(bp);
   mem = NULL;

   switch(type) {
   case GL_BYTE:
      mem = malloc(size * sizeof(GLbyte));
      break;
   case GL_UNSIGNED_BYTE:
      mem = malloc(size * sizeof(GLubyte));
      break;
   case GL_SHORT:
      mem = malloc(size * sizeof(GLshort));
      break;
   case GL_UNSIGNED_SHORT:
      mem = malloc(size * sizeof(GLushort));
      break;
   case GL_INT:
      mem = malloc(size * sizeof(GLint));
      break;
   case GL_UNSIGNED_INT:
      mem = malloc(size * sizeof(GLuint));
      break;
   case GL_FLOAT:
      mem = malloc(size * sizeof(GLfloat));
      break;
   case GL_DOUBLE:
      mem = malloc(size * sizeof(GLdouble));
      break;
   default:
      mem = NULL;
   }
   
   /*   fprintf(stderr, "Type %x ref %x mem %x size %d \r\n", 
	type, (int) ref, (int) mem, size); */

   bp = start = sdl_getbuff(sd, 4, buff);   
   if((ref = insert_mem(sd, size, type, mem)) != NULL) 
      putPointer(bp, ref); 
   
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();
   return 0;
}

int mygl_free(sdl_data *sd, int len, char *buff)
{
   char *bp;
   oglmem *mem;
   bp = ARGS(buff);
   mem = getPointer(bp);
   free_mem(sd, mem);
   return 0;
}

int mygl_read(sdl_data *sd, int len, char *buff)
{
   char *bp, *start;
   int sendlen, size, i;
   oglmem *ref;
   GLenum type;
   bp = ARGS(buff);
   type = get16be(bp);
   size = get32be(bp);
   ref  = getPointer(bp);
   switch(type) {
   case GL_BYTE:
   case GL_UNSIGNED_BYTE:
      {
	 GLbyte *mem;
	 bp = start = sdl_getbuff(sd, size*1, buff);
	 mem  = (GLbyte *) ref->ptr;
	 for(i=0; i < size; i++)
	    put8(bp, mem[i]);
      }
      break;
   case GL_SHORT:
   case GL_UNSIGNED_SHORT:
      { 
	 GLshort *mem;
	 bp = start = sdl_getbuff(sd, size*2, buff);
	 mem  = (GLshort *) ref->ptr;
	 for(i=0; i < size; i++)
	    put16be(bp, mem[i]);
      }
      break;
   case GL_INT:
   case GL_UNSIGNED_INT:
      { 
	 GLint *mem;
	 bp = start = sdl_getbuff(sd, size*4, buff);
	 mem = (GLint *) ref->ptr;
	 for(i=0; i < size; i++)
	    put32be(bp, mem[i]);
      }
      break;
   case GL_FLOAT:
      { 
	 GLfloat *mem;
	 bp = start = sdl_getbuff(sd, size*4, buff);
	 mem  = (GLfloat *) ref->ptr;
	 for(i=0; i < size; i++) {
	    putFloat32be(bp, mem[i]);
	 }
      }
      break;
   case GL_DOUBLE:
      { 
	 GLdouble *mem;
	 bp = start = sdl_getbuff(sd, size*8, buff);
	 mem  = (GLdouble *) ref->ptr;
	 for(i=0; i < size; i++) {
	    putFloat64be(bp, mem[i]);
	 }
      }
      break;
   default:
      error();
   }

   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();
   return 0;
}

int mygl_write(sdl_data *sd, int len, char *buff)
{
   unsigned char *bp, *start;
   int sendlen, size, i;
   /*   float args[] = {0.0, 1.0, 1000000.0, 0.000001, -1.0, -1000000.0, -0.000001}; 
	Testing testing */
   oglmem *ptr;
   GLenum type;
   bp = ARGS(buff);
   type = get16be(bp);
   size = get32be(bp); 
   ptr = getPointer(bp); 
   
   if(type == 1) {  /* Raw binary mode */
      GLbyte *mem;
      mem = (GLbyte *) ptr->ptr;
      memcpy(mem, bp, size);
   } else {
      switch(type) {
      case GL_BYTE:
      case GL_UNSIGNED_BYTE:
	 {
	    GLbyte *mem;
	    mem  = (GLbyte *) ptr->ptr;
	    for(i=0; i < size; i++)
	       mem[i] = get8(bp);
	 }
	 break;
      case GL_SHORT:
      case GL_UNSIGNED_SHORT:
	 { 
	    GLshort *mem;
	    mem  = (GLshort *) ptr->ptr;
	    for(i=0; i < size; i++)
	       mem[i] = get16be(bp);
	 }
	 break;
      case GL_INT:
      case GL_UNSIGNED_INT:
	 { 
	    GLint *mem;
	    mem = (GLint *) ptr->ptr;
	    for(i=0; i < size; i++)
	       mem[i] = get32be(bp);
	 }
	 break;
      case GL_FLOAT:
	 { 
	    GLfloat *mem;
	    mem  = (GLfloat *) ptr->ptr;
	    for(i=0; i < size; i++){
	       mem[i] = getFloat32be(bp);	       
	    }
	 }
	 break;
      case GL_DOUBLE:
	 { 
	    GLdouble *mem;
	    mem  = (GLdouble *) ptr->ptr;
	    for(i=0; i < size; i++) {
	       mem[i] = getFloat64be(bp);
	    }
	 }
	 break;
      default:
	 error();
      } 
   }
   bp = start = sdl_getbuff(sd, 0, buff);  /* Just sync the memcpy */
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
       error();
   return 0;
}

void free_mem(sdl_data *sd, oglmem *mem)
{
   int i;
   oglmem *temp;
   for(i =0; i < NOFFMEMREFS; i++) {
      temp = sd->memRefs[i];
      if(temp != NULL && (temp == mem)) {
	 sd->noMemRefs --;
	 sd->memRefs[i] = NULL;
	 break;
      }
   }
   if(mem->ptr != NULL)      
      free(mem->ptr);
   free(mem);
}


oglmem * insert_mem(sdl_data *sd, int size, GLenum type, char * mem) 
{
   oglmem *ref, *temp;
   int i;

   ref = malloc(sizeof(oglmem));
   if (mem != NULL && ref != NULL) 
   {
      ref->ptr = mem;
      ref->type = type;
      ref->size = size;
      if((sd->noMemRefs +1) < NOFFMEMREFS) { 
	 for(i =0; i < NOFFMEMREFS; i++) {
	    temp = sd->memRefs[i];
	    if(temp == NULL) {
	       break;
	    }
	 }
	 sd->memRefs[i] = ref;
	 sd->noMemRefs++;
	 return ref;
      }
      else  {
	 fprintf(stderr, "ESDL: Error OpenGL memory buffer overrun %d\n\r",
		 sd->noMemRefs);
	 return NULL;
      }
   }
   return NULL;
}

oglmem * esdl_util_findPointer(sdl_data *sd, void * params)
{
   oglmem * ref=NULL, *temp = NULL;
   int i;
   for(i =0; i < NOFFMEMREFS; i++) {
      temp = sd->memRefs[i];
      if(temp != NULL && (temp->ptr == params)) {
	 ref = temp;
	 break;
      }
   }
   return ref;
}

int copySdlImage2GLArray(sdl_data *sd, int len, char * buff)
{
   Uint8 *rowhi, *rowlo, type;
   SDL_Surface *image;
   oglmem * ref;
   GLubyte * mem;
   char *bp, *start;
   int i, j = 0, k, sendlen;
   Uint8  rs,bs,gs,as;

   bp = ARGS(buff);
   image = (SDL_Surface *) get32be(bp);
   ref = getPointer(bp);
   type = get8(bp);
   mem = (GLubyte *) usePointer(ref);
 
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
   rs = (2 - image->format->Rshift/8);
   gs = (2 - image->format->Gshift/8);
   bs = (2 - image->format->Bshift/8);
   as = (2 - image->format->Ashift/8);     
#else
   rs = image->format->Rshift/8;
   gs = image->format->Gshift/8;
   bs = image->format->Bshift/8;
   as = image->format->Ashift/8;     
#endif   
   /* GL surfaces are upsidedown (according to SDL examples) ?? */
   k = 0;
   rowhi = (Uint8 *)image->pixels;
   rowlo = rowhi + (image->h * image->pitch) - image->pitch;
   
   for(i=0; i<image->h; ++i ) {
      for(j=0; j<image->w; ++j ) {
	 switch(image->format->BytesPerPixel)
	 {
	 case 1:
	    mem[k++] = image->format->palette->colors[rowlo[j]].r;
	    mem[k++] = image->format->palette->colors[rowlo[j]].g;
	    mem[k++] = image->format->palette->colors[rowlo[j]].b;
	    if(type == 4)
	       mem[k++] = 0;	       
	    break;   
	 case 3:
	    mem[k++] = rowlo[j*3 + rs];
	    mem[k++] = rowlo[j*3 + gs];
	    mem[k++] = rowlo[j*3 + bs];
	    if(type == 4) 
	       mem[k++] = 0; 
	    break;
	 case 4:
	    mem[k++] = rowlo[j*3 + rs];
	    mem[k++] = rowlo[j*3 + gs];
	    mem[k++] = rowlo[j*3 + bs];
	    if(type == 4)
	       mem[k++] = rowlo[j*3 + as];
	    break;
	 }
      }
      rowlo -= image->pitch;
   }
   /*  fprintf(stderr, "i %d, j %d k%d\n\r", i, j, k); */
   bp = start = sdl_getbuff(sd, 1, buff);
   put8(bp, 1);
   sendlen = bp - start;
   if(sdl_send(sd, sendlen, buff) != sendlen)
      error();
   return 0;  
}


int sdl_util_debug(sdl_data *sd, int len, char * buff)
{
   char *bp;
   int level;
   bp = ARGS(buff);
   level = get8(bp);
   sd->debug = level;
   return 0;
}
