/*
 *  Copyright (c) 2001 Dan Gudmundsson
 *  See the file "license.terms" for information on usage and redistribution
 *  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 * 
 *     $Id: esdl_wrapper.c,v 1.9 2003/01/31 11:42:44 dgud Exp $
 */
/*    
   Command Handler
   Dispatch the work on the functions   
*/

#include <stdlib.h>

#include "esdl.h"
/* #include "esdl_opengl.h" */
#include "esdl_gl.h"
#include "esdl_glu.h"
#include "esdl_util.h"
#include "esdl_glext.h"

sdl_code_fn code_fns[] = {
   { SDL_InitFunc, "SDL_InitFunc", es_init },
   { SDL_QuitFunc, "SDL_QuitFunc", es_quit },
   { SDL_GetErrorFunc, "SDL_GetErrorFunc", es_getError },
   { SDL_SetVideoModeFunc, "SDL_SetVideoModeFunc",  es_setVideoMode },      
   { SDL_VideoDriverNameFunc, "SDL_VideoDriverNameFunc",  es_videoDriverName },      
   { SDL_GetVideoSurfaceFunc, "SDL_GetVideoSurfaceFunc",  es_getVideoSurface },
   { SDL_GetVideoInfoFunc, "SDL_GetVideoInfoFunc",     es_getVideoInfo },
   { SDL_VideoModeOKFunc, "SDL_VideoModeOKFunc",      es_videoModeOK },  
   { SDL_ListModesFunc, "SDL_ListModesFunc",        es_listModes },
   { SDL_SetColorKeyFunc, "SDL_SetColorKeyFunc",      es_setColorKey },
   { SDL_DisplayFormatFunc, "SDL_DisplayFormatFunc",    es_displayFormat },
   { SDL_BlitSurfaceFunc, "SDL_BlitSurfaceFunc",      es_blitSurface },
   { SDL_FillRectFunc, "SDL_FillRectFunc",         es_fillRect },
   { SDL_UpdateRectsFunc, "SDL_UpdateRectsFunc",      es_updateRects },
   { SDL_FlipFunc, "SDL_FlipFunc",             es_flip },
   { SDL_SetColorsFunc, "SDL_SetColorsFunc",        es_setColors },
   { SDL_MapRGBFunc, "SDL_MapRGBFunc",           es_mapRGB },
   { SDL_GetRGBFunc, "SDL_GetRGBFunc",           es_getRGB },
   { SDL_CreateRGBSurfaceFunc, "SDL_CreateRGBSurfaceFunc", es_createRGBSurface },
   { SDL_CreateRGBSurfaceFromFunc, "SDL_CreateRGBSurfaceFromFunc",  es_createRGBSurfaceFrom },
   { SDL_FreeSurfaceFunc, "SDL_FreeSurfaceFunc",      es_freeSurface },
   { SDL_LockSurfaceFunc, "SDL_LockSurfaceFunc",      es_lockSurface },
   { SDL_UnlockSurfaceFunc, "SDL_UnlockSurfaceFunc",    es_unlockSurface },
   { SDL_LoadBMPFunc, "SDL_LoadBMPFunc",    es_loadBMP },
   { SDL_SaveBMPFunc, "SDL_SaveBMPFunc",    es_saveBMP }, 
   { SDL_SetAlphaFunc, "SDL_SetAlphaFunc",    es_setAlpha }, 
   { SDL_SetClippingFunc, "SDL_SetClippingFunc",    es_setClipping },

   { SDL_SetGammaFunc,     "SDL_SetGamma",     es_setGamma },
   { SDL_SetGammaRampFunc, "SDL_SetGammaRamp", es_setGammaRamp },
   { SDL_GetGammaRampFunc, "SDL_GetGamma",     es_getGammaRamp },

   { SDL_WM_SetCaptionFunc, "SDL_WM_SetCaptionFunc", es_wm_setCaption },
   { SDL_WM_GetCaptionFunc, "SDL_WM_GetCaptionFunc", es_wm_getCaption },
   { SDL_WM_SetIconFunc, "SDL_WM_SetIconFunc",    es_wm_setIcon },
   { SDL_WM_IconifyWindowFunc, "SDL_WM_IconifyWindowFunc",    es_wm_iconifyWindow },
   { SDL_WM_ToggleFullScreenFunc, "SDL_WM_ToggleFullScreenFunc",    es_wm_toggleFullScreen },
   { SDL_WM_GrabInputFunc, "SDL_WM_GrabInputFunc",    es_wm_grabInput },
   { SDL_WM_GetInfoFunc, "SDL_WM_GetInfoFunc",     es_wm_getInfo },
   { SDL_WM_IsMaximizedFunc, "SDL_WM_IsMaximizedFunc",     es_wm_isMaximized },

   /** EVENT FUNCTIONS */
   { SDL_PumpEventsFunc, "SDL_PumpEventsFunc",  es_pumpEvents },
   { SDL_PeepEventsFunc, "SDL_PeepEventsFunc",    es_peepEvents },
   { SDL_PollEventFunc, "SDL_PollEventFunc",    es_pollEvent },
   { SDL_WaitEventFunc, "SDL_WaitEventFunc",    es_waitEvent },
   { SDL_PushEventFunc, "SDL_PushEventFunc",    es_pushEvent },
   { SDL_EventStateFunc, "SDL_EventStateFunc",    es_eventState },
   { SDL_GetAppStateFunc, "SDL_GetAppStateFunc", es_getAppState },
   { SDL_GetModStateFunc, "SDL_GetModStateFunc", es_getModState },
   { SDL_SetModStateFunc, "SDL_SetModStateFunc", es_setModState },
   { SDL_GetKeyStateFunc, "SDL_GetKeyStateFunc", es_getKeyState },
   { SDL_EnableUNICODEFunc, "SDL_EnableUNICODEFunc", es_enableUNICODE },
   { SDL_GetKeyNameFunc, "SDL_GetKeyNameFunc",    es_getKeyName },
   { SDL_EnableKeyRepeatFunc, "SDL_EnableKeyRepeatFunc",    es_enableKeyRepeat },

   /* Mouse Functions */
   { SDL_GetMouseStateFunc, "SDL_GetMouseStateFunc",  es_getMouseState },
   { SDL_GetRelativeMouseStateFunc, "SDL_GetRelativeMouseStateFunc",    es_getRelativeMouseState },
   { SDL_WarpMouseFunc, "SDL_WarpMouseFunc",    es_warpMouse },
   { SDL_CreateCursorFunc, "SDL_CreateCursorFunc",    es_createCursor },
   { SDL_SetCursorFunc, "SDL_SetCursorFunc",    es_setCursor },   
   { SDL_GetCursorFunc, "SDL_GetCursorFunc",    es_getCursor },
   { SDL_FreeCursorFunc, "SDL_FreeCursorFunc",    es_freeCursor },
   { SDL_ShowCursorFunc, "SDL_ShowCursorFunc",    es_showCursor },

   /* Joystick functions */
   { SDL_NumJoysticksFunc, "SDL_NumJoysticksFunc", es_numJoysticks },
   { SDL_JoystickNameFunc, "SDL_JoystickNameFunc", es_joystick_name },
   { SDL_JoystickOpenFunc, "SDL_JoystickOpenFunc", es_joystick_open },
   { SDL_JoystickOpenedFunc, "SDL_JoystickOpenedFunc", es_joystick_opened },
   { SDL_JoystickIndexFunc, "SDL_JoystickIndexFunc", es_joystick_index },
   { SDL_JoystickNumAxesFunc, "SDL_JoystickNumAxesFunc", es_joystick_numAxes },
   { SDL_JoystickNumBallsFunc, "SDL_JoystickNumBallsFunc", es_joystick_numBalls },
   { SDL_JoystickNumHatsFunc, "SDL_JoystickNumHatsFunc", es_joystick_numHats },
   { SDL_JoystickNumButtonsFunc, "SDL_JoystickNumButtonsFunc", es_joystick_numButtons },
   { SDL_JoystickUpdateFunc, "SDL_JoystickUpdateFunc", es_joystick_update },
   { SDL_JoystickEventStateFunc, "SDL_JoystickEventStateFunc", es_joystick_eventState },
   { SDL_JoystickGetAxisFunc, "SDL_JoystickGetAxisFunc", es_joystick_getAxis },
   { SDL_JoystickGetHatFunc, "SDL_JoystickGetHatFunc", es_joystick_getHat },
   { SDL_JoystickGetButtonFunc, "SDL_JoystickGetButtonFunc", es_joystick_getButton },
   { SDL_JoystickGetBallFunc, "SDL_JoystickGetBallFunc", es_joystick_getBall },
   { SDL_JoystickCloseFunc, "SDL_JoystickCloseFunc", es_joystick_close },

   /* SDL open gl support funcs */
   { SDL_GL_SetAttributeFunc, "SDL_GL_SetAttributeFunc", es_gl_setAttribute },
   { SDL_GL_GetAttributeFunc, "SDL_GL_GetAttributeFunc",    es_gl_getAttribute },
   { SDL_GL_SwapBuffersFunc, "SDL_GL_SwapBuffersFunc",    es_gl_swapBuffers },

   /* Erl sdl functions */
   { ESDL_getSurfaceFunc, "ESDL_getSurfaceFunc",  es_getSurface },
   { ESDL_getPixelFormatFunc, "ESDL_getPixelFormatFunc",    es_getPixelFormat },
   { ESDL_getPaletteFunc, "ESDL_getPaletteFunc",    es_getPalette },
   { ESDL_getPixelsFunc, "ESDL_getPixelsFunc",    es_getPixels },
   { SDL_UTIL_copySdlImage2GLArrayFunc, "SDL_UTIL_copySdlImage2GLArrayFunc",    copySdlImage2GLArray },
   { SDL_UTIL_DebugFunc, "SDL_UTIL_DebugFunc", sdl_util_debug},

   /* SDL AUDIO functions */
   { SDL_AudioDriverNameFunc, "SDL_AudioDriverNameFunc", es_audioDriverName },
   { SDL_OpenAudioFunc, "SDL_OpenAudioFunc", es_openAudio },
   { SDL_GetAudioStatusFunc, "SDL_GetAudioStatusFunc",    es_getAudioStatus },
   { SDL_PauseAudioFunc, "SDL_PauseAudioFunc",    es_pauseAudio },
   { SDL_LoadWAVFunc, "SDL_LoadWAVFunc",    es_loadWAV },

   /*   { SDL_LoadWAV_RWFunc, "SDL_LoadWAV_RWFunc", 
	es_loadWAV },
	    
   */
   { SDL_FreeWAVFunc, "SDL_FreeWAVFunc",  es_freeWAV },
   { SDL_BuildAudioCVTFunc, "SDL_BuildAudioCVTFunc",    es_buildAudioCVT },
   { SDL_ConvertAudioFunc, "SDL_ConvertAudioFunc",    es_convertAudio },
   { SDL_MixAudioFunc, "SDL_MixAudioFunc",    es_mixAudio },
   { SDL_LockAudioFunc, "SDL_LockAudioFunc",    es_lockAudio },
   { SDL_UnlockAudioFunc, "SDL_UnlockAudioFunc",    es_unlockAudio },
   { SDL_CloseAudioFunc, "SDL_CloseAudioFunc",    es_closeAudio },
   { play_audioFunc, "play_audioFunc",    play_audio },

   /* OPENGL 1.1 */
   { glAccumFunc,  "glAccumFunc", egl_accum },
   { glAlphaFuncFunc,  "glAlphaFuncFunc", egl_alphaFunc },
   { glAreTexturesResidentFunc,  "glAreTexturesResidentFunc", egl_areTexturesResident },
   { glArrayElementFunc,  "glArrayElementFunc", egl_arrayElement },
   { glBeginFunc,  "glBeginFunc", egl_begin },
   { glBindTextureFunc,  "glBindTextureFunc", egl_bindTexture },
   { glBitmapFunc,  "glBitmapFunc", egl_bitmap },
   { glBlendFuncFunc,  "glBlendFuncFunc", egl_blendFunc },
   { glCallListFunc,  "glCallListFunc", egl_callList },
   { glCallListsFunc,  "glCallListsFunc", egl_callLists },
   { glClearFunc,  "glClearFunc", egl_clear },
   { glClearAccumFunc,  "glClearAccumFunc", egl_clearAccum },
   { glClearColorFunc,  "glClearColorFunc", egl_clearColor },
   { glClearDepthFunc,  "glClearDepthFunc", egl_clearDepth },
   { glClearIndexFunc,  "glClearIndexFunc", egl_clearIndex },
   { glClearStencilFunc,  "glClearStencilFunc", egl_clearStencil },
   { glClipPlaneFunc,  "glClipPlaneFunc", egl_clipPlane },
   { glColor3bvFunc,  "glColor3bvFunc", egl_color3bv },
   { glColor3dvFunc,  "glColor3dvFunc", egl_color3dv },
   { glColor3fvFunc,  "glColor3fvFunc", egl_color3fv },
   { glColor3ivFunc,  "glColor3ivFunc", egl_color3iv },
   { glColor3svFunc,  "glColor3svFunc", egl_color3sv },
   { glColor3ubvFunc,  "glColor3ubvFunc", egl_color3ubv },
   { glColor3uivFunc,  "glColor3uivFunc", egl_color3uiv },
   { glColor3usvFunc,  "glColor3usvFunc", egl_color3usv },
   { glColor4bvFunc,  "glColor4bvFunc", egl_color4bv },
   { glColor4dvFunc,  "glColor4dvFunc", egl_color4dv },
   { glColor4fvFunc,  "glColor4fvFunc", egl_color4fv },
   { glColor4ivFunc,  "glColor4ivFunc", egl_color4iv },
   { glColor4svFunc,  "glColor4svFunc", egl_color4sv },
   { glColor4ubvFunc,  "glColor4ubvFunc", egl_color4ubv },
   { glColor4uivFunc,  "glColor4uivFunc", egl_color4uiv },
   { glColor4usvFunc,  "glColor4usvFunc", egl_color4usv },
   { glColorMaskFunc,  "glColorMaskFunc", egl_colorMask },
   { glColorMaterialFunc,  "glColorMaterialFunc", egl_colorMaterial },
   { glColorPointerFunc,  "glColorPointerFunc", egl_colorPointer },
   { glCopyPixelsFunc,  "glCopyPixelsFunc", egl_copyPixels },
   { glCopyTexImage1DFunc,  "glCopyTexImage1DFunc", egl_copyTexImage1D },
   { glCopyTexImage2DFunc,  "glCopyTexImage2DFunc", egl_copyTexImage2D },
   { glCopyTexSubImage1DFunc,  "glCopyTexSubImage1DFunc", egl_copyTexSubImage1D },
   { glCopyTexSubImage2DFunc,  "glCopyTexSubImage2DFunc", egl_copyTexSubImage2D },
   { glCullFaceFunc,  "glCullFaceFunc", egl_cullFace },
   { glDeleteListsFunc,  "glDeleteListsFunc", egl_deleteLists },
   { glDeleteTexturesFunc,  "glDeleteTexturesFunc", egl_deleteTextures },
   { glDepthFuncFunc,  "glDepthFuncFunc", egl_depthFunc },
   { glDepthMaskFunc,  "glDepthMaskFunc", egl_depthMask },
   { glDepthRangeFunc,  "glDepthRangeFunc", egl_depthRange },
   { glDisableFunc,  "glDisableFunc", egl_disable },
   { glDisableClientStateFunc,  "glDisableClientStateFunc", egl_disableClientState },
   { glDrawArraysFunc,  "glDrawArraysFunc", egl_drawArrays },
   { glDrawBufferFunc,  "glDrawBufferFunc", egl_drawBuffer },
   { glDrawElementsFunc,  "glDrawElementsFunc", egl_drawElements },
   { glDrawPixelsFunc,  "glDrawPixelsFunc", egl_drawPixels },
   { glEdgeFlagFunc,  "glEdgeFlagFunc", egl_edgeFlag },
   { glEdgeFlagPointerFunc,  "glEdgeFlagPointerFunc", egl_edgeFlagPointer },
   { glEdgeFlagvFunc,  "glEdgeFlagvFunc", egl_edgeFlagv },
   { glEnableFunc,  "glEnableFunc", egl_enable },
   { glEnableClientStateFunc,  "glEnableClientStateFunc", egl_enableClientState },
   { glEndFunc,  "glEndFunc", egl_end },
   { glEndListFunc,  "glEndListFunc", egl_endList },
   { glEvalCoord1dvFunc,  "glEvalCoord1dvFunc", egl_evalCoord1dv },
   { glEvalCoord1fvFunc,  "glEvalCoord1fvFunc", egl_evalCoord1fv },
   { glEvalCoord2dvFunc,  "glEvalCoord2dvFunc", egl_evalCoord2dv },
   { glEvalCoord2fvFunc,  "glEvalCoord2fvFunc", egl_evalCoord2fv },
   { glEvalMesh1Func,  "glEvalMesh1Func", egl_evalMesh1 },
   { glEvalMesh2Func,  "glEvalMesh2Func", egl_evalMesh2 },
   { glEvalPoint1Func,  "glEvalPoint1Func", egl_evalPoint1 },
   { glEvalPoint2Func,  "glEvalPoint2Func", egl_evalPoint2 },
   { glFeedbackBufferFunc,  "glFeedbackBufferFunc", egl_feedbackBuffer },
   { glFinishFunc,  "glFinishFunc", egl_finish },
   { glFlushFunc,  "glFlushFunc", egl_flush },
   { glFogfFunc,  "glFogfFunc", egl_fogf },
   { glFogfvFunc,  "glFogfvFunc", egl_fogfv },
   { glFogiFunc,  "glFogiFunc", egl_fogi },
   { glFogivFunc,  "glFogivFunc", egl_fogiv },
   { glFrontFaceFunc,  "glFrontFaceFunc", egl_frontFace },
   { glFrustumFunc,  "glFrustumFunc", egl_frustum },
   { glGenListsFunc,  "glGenListsFunc", egl_genLists },
   { glGenTexturesFunc,  "glGenTexturesFunc", egl_genTextures },
   { glGetBooleanvFunc,  "glGetBooleanvFunc", egl_getBooleanv },
   { glGetClipPlaneFunc,  "glGetClipPlaneFunc", egl_getClipPlane },
   { glGetDoublevFunc,  "glGetDoublevFunc", egl_getDoublev },
   { glGetErrorFunc,  "glGetErrorFunc", egl_getError },
   { glGetFloatvFunc,  "glGetFloatvFunc", egl_getFloatv },
   { glGetIntegervFunc,  "glGetIntegervFunc", egl_getIntegerv },
   { glGetLightfvFunc,  "glGetLightfvFunc", egl_getLightfv },
   { glGetLightivFunc,  "glGetLightivFunc", egl_getLightiv },
   { glGetMapdvFunc,  "glGetMapdvFunc", egl_getMapdv },
   { glGetMapfvFunc,  "glGetMapfvFunc", egl_getMapfv },
   { glGetMapivFunc,  "glGetMapivFunc", egl_getMapiv },
   { glGetMaterialfvFunc,  "glGetMaterialfvFunc", egl_getMaterialfv },
   { glGetMaterialivFunc,  "glGetMaterialivFunc", egl_getMaterialiv },
   { glGetPixelMapfvFunc,  "glGetPixelMapfvFunc", egl_getPixelMapfv },
   { glGetPixelMapuivFunc,  "glGetPixelMapuivFunc", egl_getPixelMapuiv },
   { glGetPixelMapusvFunc,  "glGetPixelMapusvFunc", egl_getPixelMapusv },
   { glGetPointervFunc,  "glGetPointervFunc", egl_getPointerv },
   { glGetPolygonStippleFunc,  "glGetPolygonStippleFunc", egl_getPolygonStipple },
   { glGetStringFunc,  "glGetStringFunc", egl_getString },
   { glGetTexEnvfvFunc,  "glGetTexEnvfvFunc", egl_getTexEnvfv },
   { glGetTexEnvivFunc,  "glGetTexEnvivFunc", egl_getTexEnviv },
   { glGetTexGendvFunc,  "glGetTexGendvFunc", egl_getTexGendv },
   { glGetTexGenfvFunc,  "glGetTexGenfvFunc", egl_getTexGenfv },
   { glGetTexGenivFunc,  "glGetTexGenivFunc", egl_getTexGeniv },
   { glGetTexImageFunc,  "glGetTexImageFunc", egl_getTexImage },
   { glGetTexLevelParameterfvFunc,  "glGetTexLevelParameterfvFunc", egl_getTexLevelParameterfv },
   { glGetTexLevelParameterivFunc,  "glGetTexLevelParameterivFunc", egl_getTexLevelParameteriv },
   { glGetTexParameterfvFunc,  "glGetTexParameterfvFunc", egl_getTexParameterfv },
   { glGetTexParameterivFunc,  "glGetTexParameterivFunc", egl_getTexParameteriv },
   { glHintFunc,  "glHintFunc", egl_hint },
   { glIndexMaskFunc,  "glIndexMaskFunc", egl_indexMask },
   { glIndexPointerFunc,  "glIndexPointerFunc", egl_indexPointer },
   { glIndexdFunc,  "glIndexdFunc", egl_indexd },
   { glIndexdvFunc,  "glIndexdvFunc", egl_indexdv },
   { glIndexfFunc,  "glIndexfFunc", egl_indexf },
   { glIndexfvFunc,  "glIndexfvFunc", egl_indexfv },
   { glIndexiFunc,  "glIndexiFunc", egl_indexi },
   { glIndexivFunc,  "glIndexivFunc", egl_indexiv },
   { glIndexsFunc,  "glIndexsFunc", egl_indexs },
   { glIndexsvFunc,  "glIndexsvFunc", egl_indexsv },
   { glIndexubFunc,  "glIndexubFunc", egl_indexub },
   { glIndexubvFunc,  "glIndexubvFunc", egl_indexubv },
   { glInitNamesFunc,  "glInitNamesFunc", egl_initNames },
   { glInterleavedArraysFunc,  "glInterleavedArraysFunc", egl_interleavedArrays },
   { glIsEnabledFunc,  "glIsEnabledFunc", egl_isEnabled },
   { glIsListFunc,  "glIsListFunc", egl_isList },
   { glIsTextureFunc,  "glIsTextureFunc", egl_isTexture },
   { glLightModelfFunc,  "glLightModelfFunc", egl_lightModelf },
   { glLightModelfvFunc,  "glLightModelfvFunc", egl_lightModelfv },
   { glLightModeliFunc,  "glLightModeliFunc", egl_lightModeli },
   { glLightModelivFunc,  "glLightModelivFunc", egl_lightModeliv },
   { glLightfFunc,  "glLightfFunc", egl_lightf },
   { glLightfvFunc,  "glLightfvFunc", egl_lightfv },
   { glLightiFunc,  "glLightiFunc", egl_lighti },
   { glLightivFunc,  "glLightivFunc", egl_lightiv },
   { glLineStippleFunc,  "glLineStippleFunc", egl_lineStipple },
   { glLineWidthFunc,  "glLineWidthFunc", egl_lineWidth },
   { glListBaseFunc,  "glListBaseFunc", egl_listBase },
   { glLoadIdentityFunc,  "glLoadIdentityFunc", egl_loadIdentity },
   { glLoadMatrixdFunc,  "glLoadMatrixdFunc", egl_loadMatrixd },
   { glLoadMatrixfFunc,  "glLoadMatrixfFunc", egl_loadMatrixf },
   { glLoadNameFunc,  "glLoadNameFunc", egl_loadName },
   { glLogicOpFunc,  "glLogicOpFunc", egl_logicOp },
   { glMap1dFunc,  "glMap1dFunc", egl_map1d },
   { glMap1fFunc,  "glMap1fFunc", egl_map1f },
   { glMap2dFunc,  "glMap2dFunc", egl_map2d },
   { glMap2fFunc,  "glMap2fFunc", egl_map2f },
   { glMapGrid1dFunc,  "glMapGrid1dFunc", egl_mapGrid1d },
   { glMapGrid1fFunc,  "glMapGrid1fFunc", egl_mapGrid1f },
   { glMapGrid2dFunc,  "glMapGrid2dFunc", egl_mapGrid2d },
   { glMapGrid2fFunc,  "glMapGrid2fFunc", egl_mapGrid2f },
   { glMaterialfFunc,  "glMaterialfFunc", egl_materialf },
   { glMaterialfvFunc,  "glMaterialfvFunc", egl_materialfv },
   { glMaterialiFunc,  "glMaterialiFunc", egl_materiali },
   { glMaterialivFunc,  "glMaterialivFunc", egl_materialiv },
   { glMatrixModeFunc,  "glMatrixModeFunc", egl_matrixMode },
   { glMultMatrixdFunc,  "glMultMatrixdFunc", egl_multMatrixd },
   { glMultMatrixfFunc,  "glMultMatrixfFunc", egl_multMatrixf },
   { glNewListFunc,  "glNewListFunc", egl_newList },
   { glNormal3bvFunc,  "glNormal3bvFunc", egl_normal3bv },
   { glNormal3dvFunc,  "glNormal3dvFunc", egl_normal3dv },
   { glNormal3fvFunc,  "glNormal3fvFunc", egl_normal3fv },
   { glNormal3ivFunc,  "glNormal3ivFunc", egl_normal3iv },
   { glNormal3svFunc,  "glNormal3svFunc", egl_normal3sv },
   { glNormalPointerFunc,  "glNormalPointerFunc", egl_normalPointer },
   { glOrthoFunc,  "glOrthoFunc", egl_ortho },
   { glPassThroughFunc,  "glPassThroughFunc", egl_passThrough },
   { glPixelMapfvFunc,  "glPixelMapfvFunc", egl_pixelMapfv },
   { glPixelMapuivFunc,  "glPixelMapuivFunc", egl_pixelMapuiv },
   { glPixelMapusvFunc,  "glPixelMapusvFunc", egl_pixelMapusv },
   { glPixelStorefFunc,  "glPixelStorefFunc", egl_pixelStoref },
   { glPixelStoreiFunc,  "glPixelStoreiFunc", egl_pixelStorei },
   { glPixelTransferfFunc,  "glPixelTransferfFunc", egl_pixelTransferf },
   { glPixelTransferiFunc,  "glPixelTransferiFunc", egl_pixelTransferi },
   { glPixelZoomFunc,  "glPixelZoomFunc", egl_pixelZoom },
   { glPointSizeFunc,  "glPointSizeFunc", egl_pointSize },
   { glPolygonModeFunc,  "glPolygonModeFunc", egl_polygonMode },
   { glPolygonOffsetFunc,  "glPolygonOffsetFunc", egl_polygonOffset },
   { glPolygonStippleFunc,  "glPolygonStippleFunc", egl_polygonStipple },
   { glPopAttribFunc,  "glPopAttribFunc", egl_popAttrib },
   { glPopClientAttribFunc,  "glPopClientAttribFunc", egl_popClientAttrib },
   { glPopMatrixFunc,  "glPopMatrixFunc", egl_popMatrix },
   { glPopNameFunc,  "glPopNameFunc", egl_popName },
   { glPrioritizeTexturesFunc,  "glPrioritizeTexturesFunc", egl_prioritizeTextures },
   { glPushAttribFunc,  "glPushAttribFunc", egl_pushAttrib },
   { glPushClientAttribFunc,  "glPushClientAttribFunc", egl_pushClientAttrib },
   { glPushMatrixFunc,  "glPushMatrixFunc", egl_pushMatrix },
   { glPushNameFunc,  "glPushNameFunc", egl_pushName },
   { glRasterPos2dvFunc,  "glRasterPos2dvFunc", egl_rasterPos2dv },
   { glRasterPos2fvFunc,  "glRasterPos2fvFunc", egl_rasterPos2fv },
   { glRasterPos2ivFunc,  "glRasterPos2ivFunc", egl_rasterPos2iv },
   { glRasterPos2svFunc,  "glRasterPos2svFunc", egl_rasterPos2sv },
   { glRasterPos3dvFunc,  "glRasterPos3dvFunc", egl_rasterPos3dv },
   { glRasterPos3fvFunc,  "glRasterPos3fvFunc", egl_rasterPos3fv },
   { glRasterPos3ivFunc,  "glRasterPos3ivFunc", egl_rasterPos3iv },
   { glRasterPos3svFunc,  "glRasterPos3svFunc", egl_rasterPos3sv },
   { glRasterPos4dvFunc,  "glRasterPos4dvFunc", egl_rasterPos4dv },
   { glRasterPos4fvFunc,  "glRasterPos4fvFunc", egl_rasterPos4fv },
   { glRasterPos4ivFunc,  "glRasterPos4ivFunc", egl_rasterPos4iv },
   { glRasterPos4svFunc,  "glRasterPos4svFunc", egl_rasterPos4sv },
   { glReadBufferFunc,  "glReadBufferFunc", egl_readBuffer },
   { glReadPixelsFunc,  "glReadPixelsFunc", egl_readPixels },
   { glRectdFunc,  "glRectdFunc", egl_rectd },
   { glRectdvFunc,  "glRectdvFunc", egl_rectdv },
   { glRectfFunc,  "glRectfFunc", egl_rectf },
   { glRectfvFunc,  "glRectfvFunc", egl_rectfv },
   { glRectiFunc,  "glRectiFunc", egl_recti },
   { glRectivFunc,  "glRectivFunc", egl_rectiv },
   { glRectsFunc,  "glRectsFunc", egl_rects },
   { glRectsvFunc,  "glRectsvFunc", egl_rectsv },
   { glRenderModeFunc,  "glRenderModeFunc", egl_renderMode },
   { glRotatedFunc,  "glRotatedFunc", egl_rotated },
   { glRotatefFunc,  "glRotatefFunc", egl_rotatef },
   { glScaledFunc,  "glScaledFunc", egl_scaled },
   { glScalefFunc,  "glScalefFunc", egl_scalef },
   { glScissorFunc,  "glScissorFunc", egl_scissor },
   { glSelectBufferFunc,  "glSelectBufferFunc", egl_selectBuffer },
   { glShadeModelFunc,  "glShadeModelFunc", egl_shadeModel },
   { glStencilFuncFunc,  "glStencilFuncFunc", egl_stencilFunc },
   { glStencilMaskFunc,  "glStencilMaskFunc", egl_stencilMask },
   { glStencilOpFunc,  "glStencilOpFunc", egl_stencilOp },
   { glTexCoord1dvFunc,  "glTexCoord1dvFunc", egl_texCoord1dv },
   { glTexCoord1fvFunc,  "glTexCoord1fvFunc", egl_texCoord1fv },
   { glTexCoord1ivFunc,  "glTexCoord1ivFunc", egl_texCoord1iv },
   { glTexCoord1svFunc,  "glTexCoord1svFunc", egl_texCoord1sv },
   { glTexCoord2dvFunc,  "glTexCoord2dvFunc", egl_texCoord2dv },
   { glTexCoord2fvFunc,  "glTexCoord2fvFunc", egl_texCoord2fv },
   { glTexCoord2ivFunc,  "glTexCoord2ivFunc", egl_texCoord2iv },
   { glTexCoord2svFunc,  "glTexCoord2svFunc", egl_texCoord2sv },
   { glTexCoord3dvFunc,  "glTexCoord3dvFunc", egl_texCoord3dv },
   { glTexCoord3fvFunc,  "glTexCoord3fvFunc", egl_texCoord3fv },
   { glTexCoord3ivFunc,  "glTexCoord3ivFunc", egl_texCoord3iv },
   { glTexCoord3svFunc,  "glTexCoord3svFunc", egl_texCoord3sv },
   { glTexCoord4dvFunc,  "glTexCoord4dvFunc", egl_texCoord4dv },
   { glTexCoord4fvFunc,  "glTexCoord4fvFunc", egl_texCoord4fv },
   { glTexCoord4ivFunc,  "glTexCoord4ivFunc", egl_texCoord4iv },
   { glTexCoord4svFunc,  "glTexCoord4svFunc", egl_texCoord4sv },
   { glTexCoordPointerFunc,  "glTexCoordPointerFunc", egl_texCoordPointer },
   { glTexEnvfFunc,  "glTexEnvfFunc", egl_texEnvf },
   { glTexEnvfvFunc,  "glTexEnvfvFunc", egl_texEnvfv },
   { glTexEnviFunc,  "glTexEnviFunc", egl_texEnvi },
   { glTexEnvivFunc,  "glTexEnvivFunc", egl_texEnviv },
   { glTexGendFunc,  "glTexGendFunc", egl_texGend },
   { glTexGendvFunc,  "glTexGendvFunc", egl_texGendv },
   { glTexGenfFunc,  "glTexGenfFunc", egl_texGenf },
   { glTexGenfvFunc,  "glTexGenfvFunc", egl_texGenfv },
   { glTexGeniFunc,  "glTexGeniFunc", egl_texGeni },
   { glTexGenivFunc,  "glTexGenivFunc", egl_texGeniv },
   { glTexImage1DFunc,  "glTexImage1DFunc", egl_texImage1D },
   { glTexImage2DFunc,  "glTexImage2DFunc", egl_texImage2D },
   { glTexParameterfFunc,  "glTexParameterfFunc", egl_texParameterf },
   { glTexParameterfvFunc,  "glTexParameterfvFunc", egl_texParameterfv },
   { glTexParameteriFunc,  "glTexParameteriFunc", egl_texParameteri },
   { glTexParameterivFunc,  "glTexParameterivFunc", egl_texParameteriv },
   { glTexSubImage1DFunc,  "glTexSubImage1DFunc", egl_texSubImage1D },
   { glTexSubImage2DFunc,  "glTexSubImage2DFunc", egl_texSubImage2D },
   { glTranslatedFunc,  "glTranslatedFunc", egl_translated },
   { glTranslatefFunc,  "glTranslatefFunc", egl_translatef },
   { glVertex2dvFunc,  "glVertex2dvFunc", egl_vertex2dv },
   { glVertex2fvFunc,  "glVertex2fvFunc", egl_vertex2fv },
   { glVertex2ivFunc,  "glVertex2ivFunc", egl_vertex2iv },
   { glVertex2svFunc,  "glVertex2svFunc", egl_vertex2sv },
   { glVertex3dvFunc,  "glVertex3dvFunc", egl_vertex3dv },
   { glVertex3fvFunc,  "glVertex3fvFunc", egl_vertex3fv },
   { glVertex3ivFunc,  "glVertex3ivFunc", egl_vertex3iv },
   { glVertex3svFunc,  "glVertex3svFunc", egl_vertex3sv },
   { glVertex4dvFunc,  "glVertex4dvFunc", egl_vertex4dv },
   { glVertex4fvFunc,  "glVertex4fvFunc", egl_vertex4fv },
   { glVertex4ivFunc,  "glVertex4ivFunc", egl_vertex4iv },
   { glVertex4svFunc,  "glVertex4svFunc", egl_vertex4sv },
   { glVertexPointerFunc,  "glVertexPointerFunc", egl_vertexPointer },
   { glViewportFunc,  "glViewportFunc", egl_viewport },


   /*  MY GL memory handling funcs */
   { mygl_allocFunc, "mygl_allocFunc", mygl_alloc },
   { mygl_freeFunc, "mygl_freeFunc", mygl_free },
   { mygl_readFunc, "mygl_readFunc", mygl_read },
   { mygl_writeFunc, "mygl_writeFunc", mygl_write },

   /***** GLU FUNCTIONS *****/
	
/*    /\* * * Miscellaneous functions * *\/ */
/*    { gluLookAtFunc, "gluLookAtFunc", eglu_lookAt }, */
/*    { gluOrtho2DFunc, "gluOrtho2DFunc", eglu_ortho2D }, */
/*    { gluPerspectiveFunc, "gluPerspectiveFunc", eglu_perspective }, */
/*    { gluPickMatrixFunc, "gluPickMatrixFunc", eglu_pickMatrix }, */
/*    { gluProjectFunc, "gluProjectFunc", eglu_project }, */
/*    { gluUnProjectFunc, "gluUnProjectFunc", eglu_unProject }, */
/*    { gluErrorStringFunc, "gluErrorStringFunc", eglu_errorString }, */
/*    /\* * *Mipmapping and image scaling * *\/ */
/*    { gluScaleImageFunc, "gluScaleImageFunc", eglu_scaleImage }, */
/*    { gluBuild1DMipmapsFunc, "gluBuild1DMipmapsFunc", eglu_build1DMipmaps }, */
/*    { gluBuild2DMipmapsFunc, "gluBuild2DMipmapsFunc", eglu_build2DMipmaps }, */
/*    /\* * *Quadrics * *\/ */
/*    { gluNewQuadricFunc, "gluNewQuadricFunc", eglu_newQuadric }, */
/*    { gluDeleteQuadricFunc, "gluDeleteQuadricFunc", eglu_deleteQuadric }, */
/*    { gluQuadricDrawStyleFunc, "gluQuadricDrawStyleFunc", eglu_quadricDrawStyle }, */
/*    { gluQuadricOrientationFunc, "gluQuadricOrientationFunc", eglu_quadricOrientation }, */
/*    { gluQuadricNormalsFunc, "gluQuadricNormalsFunc", eglu_quadricNormals }, */
/*    { gluQuadricTextureFunc, "gluQuadricTextureFunc", eglu_quadricTexture }, */
/*    { gluQuadricCallbackFunc, "gluQuadricCallbackFunc", eglu_quadricCallback }, */
/*    { gluCylinderFunc, "gluCylinderFunc", eglu_cylinder }, */
/*    { gluSphereFunc, "gluSphereFunc", eglu_sphere }, */
/*    { gluDiskFunc, "gluDiskFunc", eglu_disk }, */
/*    { gluPartialDiskFunc, "gluPartialDiskFunc", eglu_partialDisk }, */
/*    /\* * *Nurbs * *\/ */
/*    { gluNewNurbsRendererFunc, "gluNewNurbsRendererFunc", eglu_newNurbsRenderer }, */
/*    { gluDeleteNurbsRendererFunc, "gluDeleteNurbsRendererFunc", eglu_deleteNurbsRenderer }, */
/*    { gluLoadSamplingMatricesFunc, "gluLoadSamplingMatricesFunc", eglu_loadSamplingMatrices }, */
/*    { gluNurbsPropertyFunc, "gluNurbsPropertyFunc", eglu_nurbsProperty }, */
/*    { gluGetNurbsPropertyFunc, "gluGetNurbsPropertyFunc", eglu_getNurbsProperty }, */
/*    { gluBeginCurveFunc, "gluBeginCurveFunc", eglu_beginCurve }, */
/*    { gluEndCurveFunc, "gluEndCurveFunc", eglu_endCurve }, */
/*    { gluNurbsCurveFunc, "gluNurbsCurveFunc", eglu_nurbsCurve }, */
/*    { gluBeginSurfaceFunc, "gluBeginSurfaceFunc", eglu_beginSurface }, */
/*    { gluEndSurfaceFunc, "gluEndSurfaceFunc", eglu_endSurface }, */
/*    { gluNurbsSurfaceFunc, "gluNurbsSurfaceFunc", eglu_nurbsSurface }, */
/*    { gluBeginTrimFunc, "gluBeginTrimFunc", eglu_beginTrim }, */
/*    { gluEndTrimFunc, "gluEndTrimFunc", eglu_endTrim }, */
/*    { gluPwlCurveFunc, "gluPwlCurveFunc", eglu_pwlCurve }, */
/*    { gluNurbsCallbackFunc, "gluNurbsCallbackFunc", eglu_nurbsCallback }, */
/*    /\* * *Polygon tessellation * *\/ */
/*    { gluNewTessFunc, "gluNewTessFunc", eglu_newTess }, */
/*    { gluDeleteTessFunc, "gluDeleteTessFunc", eglu_deleteTess }, */
/*    { gluTessBeginPolygonFunc, "gluTessBeginPolygonFunc", eglu_tessBeginPolygon }, */
/*    { gluTessBeginContourFunc, "gluTessBeginContourFunc", eglu_tessBeginContour }, */
/*    { gluTessVertexFunc, "gluTessVertexFunc", eglu_tessVertex }, */
/*    { gluTessEndContourFunc, "gluTessEndContourFunc", eglu_tessEndContour }, */
/*    { gluTessEndPolygonFunc, "gluTessEndPolygonFunc", eglu_tessEndPolygon }, */
/*    { gluTessPropertyFunc, "gluTessPropertyFunc", eglu_tessProperty }, */
/*    { gluTessNormalFunc, "gluTessNormalFunc", eglu_tessNormal }, */
/*    { gluTessCallbackFunc, "gluTessCallbackFunc", eglu_tessCallback }, */
/*    { gluGetTessPropertyFunc, "gluGetTessPropertyFunc", eglu_getTessProperty }, */
/*    /\* * *Obsolete 1.0 tessellation functions * *\/ */
/*    { gluBeginPolygonFunc, "gluBeginPolygonFunc", eglu_beginPolygon }, */
/*    { gluNextContourFunc, "gluNextContourFunc", eglu_nextContour }, */
/*    { gluEndPolygonFunc, "gluEndPolygonFunc", eglu_endPolygon }, */
/*    /\* * *New functions in GLU 1.1 * *\/ */
/*    { gluGetStringFunc, "gluGetStringFunc", eglu_getString },/\* * *GLU 1.3 functions * *\/ */
/*    { gluCheckExtensionFunc, "gluCheckExtensionFunc", eglu_checkExtension },    */

   { gluBeginCurveFunc,  "gluBeginCurve", eglu_beginCurve },
   { gluBeginPolygonFunc,  "gluBeginPolygon", eglu_beginPolygon },
   { gluBeginSurfaceFunc,  "gluBeginSurface", eglu_beginSurface },
   { gluBeginTrimFunc,  "gluBeginTrim", eglu_beginTrim },
   { gluBuild1DMipmapsFunc,  "gluBuild1DMipmaps", eglu_build1DMipmaps },
   { gluBuild2DMipmapsFunc,  "gluBuild2DMipmaps", eglu_build2DMipmaps },
/*    { gluBuild3DMipmapsFunc,  "gluBuild3DMipmaps", eglu_build3DMipmaps }, */
   { gluCylinderFunc,  "gluCylinder", eglu_cylinder },
   { gluDeleteNurbsRendererFunc,  "gluDeleteNurbsRenderer", eglu_deleteNurbsRenderer },
   { gluDeleteQuadricFunc,  "gluDeleteQuadric", eglu_deleteQuadric },
   { gluDeleteTessFunc,  "gluDeleteTess", eglu_deleteTess },
   { gluDiskFunc,  "gluDisk", eglu_disk },
   { gluEndCurveFunc,  "gluEndCurve", eglu_endCurve },
   { gluEndPolygonFunc,  "gluEndPolygon", eglu_endPolygon },
   { gluEndSurfaceFunc,  "gluEndSurface", eglu_endSurface },
   { gluEndTrimFunc,  "gluEndTrim", eglu_endTrim },
   { gluErrorStringFunc,  "gluErrorString", eglu_errorString },
   { gluGetNurbsPropertyFunc,  "gluGetNurbsProperty", eglu_getNurbsProperty },
   { gluGetStringFunc,  "gluGetString", eglu_getString },
   { gluGetTessPropertyFunc,  "gluGetTessProperty", eglu_getTessProperty },
   { gluLoadSamplingMatricesFunc,  "gluLoadSamplingMatrices", eglu_loadSamplingMatrices },
   { gluLookAtFunc,  "gluLookAt", eglu_lookAt },
   { gluNewNurbsRendererFunc,  "gluNewNurbsRenderer", eglu_newNurbsRenderer },
   { gluNewQuadricFunc,  "gluNewQuadric", eglu_newQuadric },
   { gluNewTessFunc,  "gluNewTess", eglu_newTess },
   { gluNextContourFunc,  "gluNextContour", eglu_nextContour },
   { gluNurbsCurveFunc,  "gluNurbsCurve", eglu_nurbsCurve },
   { gluNurbsPropertyFunc,  "gluNurbsProperty", eglu_nurbsProperty },
   { gluNurbsSurfaceFunc,  "gluNurbsSurface", eglu_nurbsSurface },
   { gluOrtho2DFunc,  "gluOrtho2D", eglu_ortho2D },
   { gluPartialDiskFunc,  "gluPartialDisk", eglu_partialDisk },
   { gluPerspectiveFunc,  "gluPerspective", eglu_perspective },
   { gluPickMatrixFunc,  "gluPickMatrix", eglu_pickMatrix },
   { gluProjectFunc,  "gluProject", eglu_project },
   { gluPwlCurveFunc,  "gluPwlCurve", eglu_pwlCurve },
   { gluQuadricDrawStyleFunc,  "gluQuadricDrawStyle", eglu_quadricDrawStyle },
   { gluQuadricNormalsFunc,  "gluQuadricNormals", eglu_quadricNormals },
   { gluQuadricOrientationFunc,  "gluQuadricOrientation", eglu_quadricOrientation },
   { gluQuadricTextureFunc,  "gluQuadricTexture", eglu_quadricTexture },
   { gluScaleImageFunc,  "gluScaleImage", eglu_scaleImage },
   { gluSphereFunc,  "gluSphere", eglu_sphere },
   { gluTessBeginContourFunc,  "gluTessBeginContour", eglu_tessBeginContour },
   { gluTessBeginPolygonFunc,  "gluTessBeginPolygon", eglu_tessBeginPolygon },
   { gluTessCallbackFunc,  "gluTessCallback", eglu_tessCallback },
   { gluTessEndContourFunc,  "gluTessEndContour", eglu_tessEndContour },
   { gluTessEndPolygonFunc,  "gluTessEndPolygon", eglu_tessEndPolygon },
   { gluTessNormalFunc,  "gluTessNormal", eglu_tessNormal },
   { gluTessPropertyFunc,  "gluTessProperty", eglu_tessProperty },
   { gluTessVertexFunc,  "gluTessVertex", eglu_tessVertex },
   { gluUnProjectFunc,  "gluUnProject", eglu_unProject },

   /*  Opengl 1.2 or later with ARB's and requested extensions  */
   { glBlendColorFunc,  "glBlendColor", esdl_load_glBlendColor },
   { glBlendEquationFunc,  "glBlendEquation", esdl_load_glBlendEquation },
   { glDrawRangeElementsFunc,  "glDrawRangeElements", esdl_load_glDrawRangeElements },
   { glColorTableFunc,  "glColorTable", esdl_load_glColorTable },
   { glColorTableParameterfvFunc,  "glColorTableParameterfv", esdl_load_glColorTableParameterfv },
   { glColorTableParameterivFunc,  "glColorTableParameteriv", esdl_load_glColorTableParameteriv },
   { glCopyColorTableFunc,  "glCopyColorTable", esdl_load_glCopyColorTable },
   { glGetColorTableFunc,  "glGetColorTable", esdl_load_glGetColorTable },
   { glGetColorTableParameterfvFunc,  "glGetColorTableParameterfv", esdl_load_glGetColorTableParameterfv },
   { glGetColorTableParameterivFunc,  "glGetColorTableParameteriv", esdl_load_glGetColorTableParameteriv },
   { glColorSubTableFunc,  "glColorSubTable", esdl_load_glColorSubTable },
   { glCopyColorSubTableFunc,  "glCopyColorSubTable", esdl_load_glCopyColorSubTable },
   { glConvolutionFilter1DFunc,  "glConvolutionFilter1D", esdl_load_glConvolutionFilter1D },
   { glConvolutionFilter2DFunc,  "glConvolutionFilter2D", esdl_load_glConvolutionFilter2D },
   { glConvolutionParameterfFunc,  "glConvolutionParameterf", esdl_load_glConvolutionParameterf },
   { glConvolutionParameterfvFunc,  "glConvolutionParameterfv", esdl_load_glConvolutionParameterfv },
   { glConvolutionParameteriFunc,  "glConvolutionParameteri", esdl_load_glConvolutionParameteri },
   { glConvolutionParameterivFunc,  "glConvolutionParameteriv", esdl_load_glConvolutionParameteriv },
   { glCopyConvolutionFilter1DFunc,  "glCopyConvolutionFilter1D", esdl_load_glCopyConvolutionFilter1D },
   { glCopyConvolutionFilter2DFunc,  "glCopyConvolutionFilter2D", esdl_load_glCopyConvolutionFilter2D },
   { glGetConvolutionFilterFunc,  "glGetConvolutionFilter", esdl_load_glGetConvolutionFilter },
   { glGetConvolutionParameterfvFunc,  "glGetConvolutionParameterfv", esdl_load_glGetConvolutionParameterfv },
   { glGetConvolutionParameterivFunc,  "glGetConvolutionParameteriv", esdl_load_glGetConvolutionParameteriv },
   { glGetSeparableFilterFunc,  "glGetSeparableFilter", esdl_load_glGetSeparableFilter },
   { glSeparableFilter2DFunc,  "glSeparableFilter2D", esdl_load_glSeparableFilter2D },
   { glGetHistogramFunc,  "glGetHistogram", esdl_load_glGetHistogram },
   { glGetHistogramParameterfvFunc,  "glGetHistogramParameterfv", esdl_load_glGetHistogramParameterfv },
   { glGetHistogramParameterivFunc,  "glGetHistogramParameteriv", esdl_load_glGetHistogramParameteriv },
   { glGetMinmaxFunc,  "glGetMinmax", esdl_load_glGetMinmax },
   { glGetMinmaxParameterfvFunc,  "glGetMinmaxParameterfv", esdl_load_glGetMinmaxParameterfv },
   { glGetMinmaxParameterivFunc,  "glGetMinmaxParameteriv", esdl_load_glGetMinmaxParameteriv },
   { glHistogramFunc,  "glHistogram", esdl_load_glHistogram },
   { glMinmaxFunc,  "glMinmax", esdl_load_glMinmax },
   { glResetHistogramFunc,  "glResetHistogram", esdl_load_glResetHistogram },
   { glResetMinmaxFunc,  "glResetMinmax", esdl_load_glResetMinmax },
   { glTexImage3DFunc,  "glTexImage3D", esdl_load_glTexImage3D },
   { glTexSubImage3DFunc,  "glTexSubImage3D", esdl_load_glTexSubImage3D },
   { glCopyTexSubImage3DFunc,  "glCopyTexSubImage3D", esdl_load_glCopyTexSubImage3D },
   { glActiveTextureFunc,  "glActiveTexture", esdl_load_glActiveTexture },
   { glClientActiveTextureFunc,  "glClientActiveTexture", esdl_load_glClientActiveTexture },
   { glMultiTexCoord1dvFunc,  "glMultiTexCoord1dv", esdl_load_glMultiTexCoord1dv },
   { glMultiTexCoord1fvFunc,  "glMultiTexCoord1fv", esdl_load_glMultiTexCoord1fv },
   { glMultiTexCoord1ivFunc,  "glMultiTexCoord1iv", esdl_load_glMultiTexCoord1iv },
   { glMultiTexCoord1svFunc,  "glMultiTexCoord1sv", esdl_load_glMultiTexCoord1sv },
   { glMultiTexCoord2dvFunc,  "glMultiTexCoord2dv", esdl_load_glMultiTexCoord2dv },
   { glMultiTexCoord2fvFunc,  "glMultiTexCoord2fv", esdl_load_glMultiTexCoord2fv },
   { glMultiTexCoord2ivFunc,  "glMultiTexCoord2iv", esdl_load_glMultiTexCoord2iv },
   { glMultiTexCoord2svFunc,  "glMultiTexCoord2sv", esdl_load_glMultiTexCoord2sv },
   { glMultiTexCoord3dvFunc,  "glMultiTexCoord3dv", esdl_load_glMultiTexCoord3dv },
   { glMultiTexCoord3fvFunc,  "glMultiTexCoord3fv", esdl_load_glMultiTexCoord3fv },
   { glMultiTexCoord3ivFunc,  "glMultiTexCoord3iv", esdl_load_glMultiTexCoord3iv },
   { glMultiTexCoord3svFunc,  "glMultiTexCoord3sv", esdl_load_glMultiTexCoord3sv },
   { glMultiTexCoord4dvFunc,  "glMultiTexCoord4dv", esdl_load_glMultiTexCoord4dv },
   { glMultiTexCoord4fvFunc,  "glMultiTexCoord4fv", esdl_load_glMultiTexCoord4fv },
   { glMultiTexCoord4ivFunc,  "glMultiTexCoord4iv", esdl_load_glMultiTexCoord4iv },
   { glMultiTexCoord4svFunc,  "glMultiTexCoord4sv", esdl_load_glMultiTexCoord4sv },
   { glLoadTransposeMatrixfFunc,  "glLoadTransposeMatrixf", esdl_load_glLoadTransposeMatrixf },
   { glLoadTransposeMatrixdFunc,  "glLoadTransposeMatrixd", esdl_load_glLoadTransposeMatrixd },
   { glMultTransposeMatrixfFunc,  "glMultTransposeMatrixf", esdl_load_glMultTransposeMatrixf },
   { glMultTransposeMatrixdFunc,  "glMultTransposeMatrixd", esdl_load_glMultTransposeMatrixd },
   { glSampleCoverageFunc,  "glSampleCoverage", esdl_load_glSampleCoverage },
   { glCompressedTexImage3DFunc,  "glCompressedTexImage3D", esdl_load_glCompressedTexImage3D },
   { glCompressedTexImage2DFunc,  "glCompressedTexImage2D", esdl_load_glCompressedTexImage2D },
   { glCompressedTexImage1DFunc,  "glCompressedTexImage1D", esdl_load_glCompressedTexImage1D },
   { glCompressedTexSubImage3DFunc,  "glCompressedTexSubImage3D", esdl_load_glCompressedTexSubImage3D },
   { glCompressedTexSubImage2DFunc,  "glCompressedTexSubImage2D", esdl_load_glCompressedTexSubImage2D },
   { glCompressedTexSubImage1DFunc,  "glCompressedTexSubImage1D", esdl_load_glCompressedTexSubImage1D },
   { glGetCompressedTexImageFunc,  "glGetCompressedTexImage", esdl_load_glGetCompressedTexImage },
   { glBlendFuncSeparateFunc,  "glBlendFuncSeparate", esdl_load_glBlendFuncSeparate },
   { glFogCoordfFunc,  "glFogCoordf", esdl_load_glFogCoordf },
   { glFogCoordfvFunc,  "glFogCoordfv", esdl_load_glFogCoordfv },
   { glFogCoorddFunc,  "glFogCoordd", esdl_load_glFogCoordd },
   { glFogCoorddvFunc,  "glFogCoorddv", esdl_load_glFogCoorddv },
   { glFogCoordPointerFunc,  "glFogCoordPointer", esdl_load_glFogCoordPointer },
   { glMultiDrawArraysFunc,  "glMultiDrawArrays", esdl_load_glMultiDrawArrays },
   { glPointParameterfFunc,  "glPointParameterf", esdl_load_glPointParameterf },
   { glPointParameterfvFunc,  "glPointParameterfv", esdl_load_glPointParameterfv },
   { glPointParameteriFunc,  "glPointParameteri", esdl_load_glPointParameteri },
   { glPointParameterivFunc,  "glPointParameteriv", esdl_load_glPointParameteriv },
   { glSecondaryColor3bvFunc,  "glSecondaryColor3bv", esdl_load_glSecondaryColor3bv },
   { glSecondaryColor3dvFunc,  "glSecondaryColor3dv", esdl_load_glSecondaryColor3dv },
   { glSecondaryColor3fvFunc,  "glSecondaryColor3fv", esdl_load_glSecondaryColor3fv },
   { glSecondaryColor3ivFunc,  "glSecondaryColor3iv", esdl_load_glSecondaryColor3iv },
   { glSecondaryColor3svFunc,  "glSecondaryColor3sv", esdl_load_glSecondaryColor3sv },
   { glSecondaryColor3ubvFunc,  "glSecondaryColor3ubv", esdl_load_glSecondaryColor3ubv },
   { glSecondaryColor3uivFunc,  "glSecondaryColor3uiv", esdl_load_glSecondaryColor3uiv },
   { glSecondaryColor3usvFunc,  "glSecondaryColor3usv", esdl_load_glSecondaryColor3usv },
   { glSecondaryColorPointerFunc,  "glSecondaryColorPointer", esdl_load_glSecondaryColorPointer },
   { glWindowPos2dvFunc,  "glWindowPos2dv", esdl_load_glWindowPos2dv },
   { glWindowPos2fvFunc,  "glWindowPos2fv", esdl_load_glWindowPos2fv },
   { glWindowPos2ivFunc,  "glWindowPos2iv", esdl_load_glWindowPos2iv },
   { glWindowPos2svFunc,  "glWindowPos2sv", esdl_load_glWindowPos2sv },
   { glWindowPos3dvFunc,  "glWindowPos3dv", esdl_load_glWindowPos3dv },
   { glWindowPos3fvFunc,  "glWindowPos3fv", esdl_load_glWindowPos3fv },
   { glWindowPos3ivFunc,  "glWindowPos3iv", esdl_load_glWindowPos3iv },
   { glWindowPos3svFunc,  "glWindowPos3sv", esdl_load_glWindowPos3sv },
   { glWeightbvARBFunc,  "glWeightbvARB", esdl_load_glWeightbvARB },
   { glWeightsvARBFunc,  "glWeightsvARB", esdl_load_glWeightsvARB },
   { glWeightivARBFunc,  "glWeightivARB", esdl_load_glWeightivARB },
   { glWeightfvARBFunc,  "glWeightfvARB", esdl_load_glWeightfvARB },
   { glWeightdvARBFunc,  "glWeightdvARB", esdl_load_glWeightdvARB },
   { glWeightubvARBFunc,  "glWeightubvARB", esdl_load_glWeightubvARB },
   { glWeightusvARBFunc,  "glWeightusvARB", esdl_load_glWeightusvARB },
   { glWeightuivARBFunc,  "glWeightuivARB", esdl_load_glWeightuivARB },
   { glWeightPointerARBFunc,  "glWeightPointerARB", esdl_load_glWeightPointerARB },
   { glVertexBlendARBFunc,  "glVertexBlendARB", esdl_load_glVertexBlendARB },
   { glCurrentPaletteMatrixARBFunc,  "glCurrentPaletteMatrixARB", esdl_load_glCurrentPaletteMatrixARB },
   { glMatrixIndexubvARBFunc,  "glMatrixIndexubvARB", esdl_load_glMatrixIndexubvARB },
   { glMatrixIndexusvARBFunc,  "glMatrixIndexusvARB", esdl_load_glMatrixIndexusvARB },
   { glMatrixIndexuivARBFunc,  "glMatrixIndexuivARB", esdl_load_glMatrixIndexuivARB },
   { glMatrixIndexPointerARBFunc,  "glMatrixIndexPointerARB", esdl_load_glMatrixIndexPointerARB },
   { glVertexAttrib1dvARBFunc,  "glVertexAttrib1dvARB", esdl_load_glVertexAttrib1dvARB },
   { glVertexAttrib1fvARBFunc,  "glVertexAttrib1fvARB", esdl_load_glVertexAttrib1fvARB },
   { glVertexAttrib1svARBFunc,  "glVertexAttrib1svARB", esdl_load_glVertexAttrib1svARB },
   { glVertexAttrib2dvARBFunc,  "glVertexAttrib2dvARB", esdl_load_glVertexAttrib2dvARB },
   { glVertexAttrib2fvARBFunc,  "glVertexAttrib2fvARB", esdl_load_glVertexAttrib2fvARB },
   { glVertexAttrib2svARBFunc,  "glVertexAttrib2svARB", esdl_load_glVertexAttrib2svARB },
   { glVertexAttrib3dvARBFunc,  "glVertexAttrib3dvARB", esdl_load_glVertexAttrib3dvARB },
   { glVertexAttrib3fvARBFunc,  "glVertexAttrib3fvARB", esdl_load_glVertexAttrib3fvARB },
   { glVertexAttrib3svARBFunc,  "glVertexAttrib3svARB", esdl_load_glVertexAttrib3svARB },
   { glVertexAttrib4NbvARBFunc,  "glVertexAttrib4NbvARB", esdl_load_glVertexAttrib4NbvARB },
   { glVertexAttrib4NivARBFunc,  "glVertexAttrib4NivARB", esdl_load_glVertexAttrib4NivARB },
   { glVertexAttrib4NsvARBFunc,  "glVertexAttrib4NsvARB", esdl_load_glVertexAttrib4NsvARB },
   { glVertexAttrib4NubARBFunc,  "glVertexAttrib4NubARB", esdl_load_glVertexAttrib4NubARB },
   { glVertexAttrib4NubvARBFunc,  "glVertexAttrib4NubvARB", esdl_load_glVertexAttrib4NubvARB },
   { glVertexAttrib4NuivARBFunc,  "glVertexAttrib4NuivARB", esdl_load_glVertexAttrib4NuivARB },
   { glVertexAttrib4NusvARBFunc,  "glVertexAttrib4NusvARB", esdl_load_glVertexAttrib4NusvARB },
   { glVertexAttrib4bvARBFunc,  "glVertexAttrib4bvARB", esdl_load_glVertexAttrib4bvARB },
   { glVertexAttrib4dvARBFunc,  "glVertexAttrib4dvARB", esdl_load_glVertexAttrib4dvARB },
   { glVertexAttrib4fvARBFunc,  "glVertexAttrib4fvARB", esdl_load_glVertexAttrib4fvARB },
   { glVertexAttrib4ivARBFunc,  "glVertexAttrib4ivARB", esdl_load_glVertexAttrib4ivARB },
   { glVertexAttrib4svARBFunc,  "glVertexAttrib4svARB", esdl_load_glVertexAttrib4svARB },
   { glVertexAttrib4ubvARBFunc,  "glVertexAttrib4ubvARB", esdl_load_glVertexAttrib4ubvARB },
   { glVertexAttrib4uivARBFunc,  "glVertexAttrib4uivARB", esdl_load_glVertexAttrib4uivARB },
   { glVertexAttrib4usvARBFunc,  "glVertexAttrib4usvARB", esdl_load_glVertexAttrib4usvARB },
   { glVertexAttribPointerARBFunc,  "glVertexAttribPointerARB", esdl_load_glVertexAttribPointerARB },
   { glEnableVertexAttribArrayARBFunc,  "glEnableVertexAttribArrayARB", esdl_load_glEnableVertexAttribArrayARB },
   { glDisableVertexAttribArrayARBFunc,  "glDisableVertexAttribArrayARB", esdl_load_glDisableVertexAttribArrayARB },
   { glProgramStringARBFunc,  "glProgramStringARB", esdl_load_glProgramStringARB },
   { glBindProgramARBFunc,  "glBindProgramARB", esdl_load_glBindProgramARB },
   { glDeleteProgramsARBFunc,  "glDeleteProgramsARB", esdl_load_glDeleteProgramsARB },
   { glGenProgramsARBFunc,  "glGenProgramsARB", esdl_load_glGenProgramsARB },
   { glProgramEnvParameter4dvARBFunc,  "glProgramEnvParameter4dvARB", esdl_load_glProgramEnvParameter4dvARB },
   { glProgramEnvParameter4fvARBFunc,  "glProgramEnvParameter4fvARB", esdl_load_glProgramEnvParameter4fvARB },
   { glProgramLocalParameter4dvARBFunc,  "glProgramLocalParameter4dvARB", esdl_load_glProgramLocalParameter4dvARB },
   { glProgramLocalParameter4fvARBFunc,  "glProgramLocalParameter4fvARB", esdl_load_glProgramLocalParameter4fvARB },
   { glGetProgramEnvParameterdvARBFunc,  "glGetProgramEnvParameterdvARB", esdl_load_glGetProgramEnvParameterdvARB },
   { glGetProgramEnvParameterfvARBFunc,  "glGetProgramEnvParameterfvARB", esdl_load_glGetProgramEnvParameterfvARB },
   { glGetProgramLocalParameterdvARBFunc,  "glGetProgramLocalParameterdvARB", esdl_load_glGetProgramLocalParameterdvARB },
   { glGetProgramLocalParameterfvARBFunc,  "glGetProgramLocalParameterfvARB", esdl_load_glGetProgramLocalParameterfvARB },
   { glGetProgramivARBFunc,  "glGetProgramivARB", esdl_load_glGetProgramivARB },
   { glGetProgramStringARBFunc,  "glGetProgramStringARB", esdl_load_glGetProgramStringARB },
   { glGetVertexAttribdvARBFunc,  "glGetVertexAttribdvARB", esdl_load_glGetVertexAttribdvARB },
   { glGetVertexAttribfvARBFunc,  "glGetVertexAttribfvARB", esdl_load_glGetVertexAttribfvARB },
   { glGetVertexAttribivARBFunc,  "glGetVertexAttribivARB", esdl_load_glGetVertexAttribivARB },
   { glGetVertexAttribPointervARBFunc,  "glGetVertexAttribPointervARB", esdl_load_glGetVertexAttribPointervARB },
   { glIsProgramARBFunc,  "glIsProgramARB", esdl_load_glIsProgramARB },

   { 0, "LastFunction", NULL}} ;

sdl_code_fn ** init_fps()
{   
   int i = 0;
   int n = MAX_FUNCTIONS_H;  /* Must be greater than then the last function op */ 
   sdl_code_fn **fps;
   
   
   fps = malloc(sizeof(sdl_code_fn *) * n+1);
   if(fps == NULL)
       return NULL;
   
   for(i=0; i<n; i++)
       fps[i] = NULL;

   i = 0;
   while(code_fns[i].op != 0 ) 
   {
      if(fps[code_fns[i].op] == NULL && i < OPENGL_EXTS_H)
	 fps[code_fns[i].op] = &code_fns[i];
      else {
	 if(i < OPENGL_EXTS_H) 
	    fprintf(stderr, 
		    "FParray mismatch in initialization: %d %s %d %s\r\n", 
		    i, fps[code_fns[i].op]->str, code_fns[i].op, code_fns[i].str);
      }
      i++;
   }
   return fps;
}


/* Must done after a rendering context is created */
void init_glexts(sdl_code_fn ** fps) 
{
   int i = 0;
   static int already_done = 0;
   if(already_done == 0) 
   {
      /* fprintf(stderr, "\r\nInit GL EXTS \r\n");*/
      while(code_fns[i].op < OPENGL_EXTS_H)
	 i++;
      while(code_fns[i].op != 0) 
      {
	 if(fps[code_fns[i].op] != NULL) {
	    sdl_fun ptr;
	    if((ptr = ((sdl_load_fun)(code_fns[i].fn))()) != NULL) {
	       /* fprintf(stderr, "Success %s \r\n", code_fns[i].str);*/
	       code_fns[i].fn = ptr;
	       fps[code_fns[i].op] = &code_fns[i];
	    } else {
	       /* fprintf(stderr, "Failed %s \r\n", code_fns[i].str); */
	       fps[code_fns[i].op] = NULL;
	    }
	 }
	 else {	 
	    fprintf(stderr, "Exiting FP EXTENSION array mismatch in initialization %d %d %s\r\n", 
		    i, code_fns[i].op, code_fns[i].str);
	 }
	 i++;
      }
      already_done = 1;
   }
}

int do_command(sdl_data * sd, unsigned int command, int len, char * buff)
{   
   int op;
   char* tmp;

   tmp = buff;
   if(command == 0) {
      op = get16be(tmp);
   } else {
      op = command;
   }

   sd->op = op;
   if(sd->debug) {
      fprintf(stderr, "Command(%d: ", op);
      if (sd->fns[op] != NULL && sd->fns[op]->fn != NULL) {
	 fprintf(stderr, "%s): %x %d ", 
		 sd->fns[op]->str, (int) sd->fns[op], len);
	 (sd->fns[op]->fn)(sd, len, buff);
	 fprintf(stderr, "done \r\n");
	 return 1;
      }
      else {
	 fprintf(stderr, "No function found (maybe missing opengl extension) \r\n");
	 return 0;
     }
   } else {  
      if (sd->fns[op] == NULL) {
	  return 0;
      } else {
	  (sd->fns[op]->fn)(sd, len, buff);
	  return 1;
      }
   }
}
