%%  Copyright (c) 2001 Dan Gudmundsson
%%
%%  See the file "license.terms" for information on usage and redistribution
%%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
%% 
%%     $Id: sdl_mouse.erl,v 1.4 2003/01/13 14:37:14 dgud Exp $
%%
%%%----------------------------------------------------------------------
%%% File    : sdl_mouse.erl
%%% Author  : Dan Gudmundsson <dgud@erix.ericsson.se>
%%% Purpose : Mouse related functions
%%% Created : 12 Jul 2000 by Dan Gudmundsson <dgud@erix.ericsson.se>
%%%----------------------------------------------------------------------

-module(sdl_mouse).
-author('dgud@erix.ericsson.se').

-include("esdl.hrl").
-include("sdl_mouse.hrl").

%%-compile(export_all).
-export([
	 %% Mouse Related 
	 createCursor/6,
	 freeCursor/1,
	 getCursor/0,
	 getMouseState/0, 
	 getRelativeMouseState/0,
	 setCursor/1,
	 showCursor/1,
	 warpMouse/2
	]).

-import(sdl, [call/1, cast/1]).

-define(SDL_GetMouseState, ?SDL_MOUSE_HRL+1).
-define(SDL_GetRelativeMouseState, ?SDL_GetMouseState+1).
-define(SDL_WarpMouse, ?SDL_GetRelativeMouseState +1).
-define(SDL_CreateCursor, ?SDL_WarpMouse +1).
-define(SDL_SetCursor,  ?SDL_CreateCursor+1).
-define(SDL_GetCursor,  ?SDL_SetCursor+1).
-define(SDL_FreeCursor, ?SDL_GetCursor+1).
-define(SDL_ShowCursor, ?SDL_FreeCursor+1).

%%%%%%%%%%%%%%%%%%%%% MOUSE FUNCTIONS %%%%%%%%%%%%%%%%%%%%%%%%%%
%% Func:  getMouseState
%% Args:  none
%% Returns:  {MouseState, X, Y}
%% C-API func:  Uint8 SDL_GetMouseState(int *x, int *y);
%% Desc:  
getMouseState() ->
    <<State:8, X:16, Y:16>> = call( <<(?SDL_GetMouseState):16>>),
    {State, X, Y}.

%% Func:  getRelativeMouseState
%% Args:  none
%% Returns:  {MouseState, X, Y}
%% C-API func: Uint8 SDL_GetRelativeMouseState(int *x, int *y);
%% Desc:  
getRelativeMouseState() ->
    <<State:8, X:16, Y:16>> = call( <<(?SDL_GetMouseState):16>>),
    {State, X, Y}.

%% Func:  warpMouse
%% Args:  X, Y
%% Returns:  ok 
%% C-API func: void SDL_WarpMouse(Uint16 x, Uint16 y);
%% Desc:  
warpMouse(X, Y) ->
    cast( <<(?SDL_WarpMouse):16, X:16, Y:16>>),
    ok.

%% Func:  createCursor
%% Args:  Data (Binary), Mask(Binary), W, H, HotX, HotY
%% Returns:  CursorRef
%% C-API func:  SDL_Cursor *SDL_CreateCursor
%%		(Uint8 *data, Uint8 *mask, int w, int h, int hot_x, int hot_y);
%% Desc:  Data & Mask must be less than 32*32 bytes.
createCursor(Data, Mask, W, H, HotX, HotY) when binary(Data), binary(Mask) ->
    case call( <<(?SDL_CreateCursor):16,(size(Data)):16, Data/binary,
	       (size(Mask)):16, Mask/binary, 
	       W:16, H:16, HotX:16, HotY:16>>) of
	<<0:32>> ->
	    exit({createCursor, returned_null});
	<<Ptr:32>> ->
	    {cursorp, Ptr}
    end.

%% Func:  setCursor
%% Args:  CursorRef
%% Returns:  ok
%% C-API func:  void SDL_SetCursor(SDL_Cursor *cursor);
setCursor({cursorp, Ref}) -> 
    cast( <<(?SDL_SetCursor):16, Ref:32>>),
    ok.

%% Func:  getCursor
%% Args:  none
%% Returns:  A cursorRef
%% C-API func:  void SDL_SetCursor(SDL_Cursor *cursor);
getCursor() ->  
    case call( <<(?SDL_GetCursor):16>>) of
	<<0:32>> ->
	    exit({getCursor, returned_null});
	<<Ptr:32>> ->
	    {cursorp, Ptr}
    end.

%% Func:  freeCursor
%% Args:  CursorRef
%% Returns:  ok
%% C-API func:  void SDL_FreeCursor(SDL_Cursor *cursor);
freeCursor({cursorp, Ref}) -> 
    cast( <<(?SDL_FreeCursor):16, Ref:32>>),
    ok.

%% Func:  showCursor
%% Args:  true | false
%% Returns:  true | false (if cursor was displayed before the call)
%% C-API func:  int SDL_ShowCursor(int toggle);
showCursor(Bool) -> 
    B = if 
	    Bool == true -> 1;
	    true -> 0
	end,
    <<Res:8>> = call( <<(?SDL_ShowCursor):16,B:8>>),
    Res == 1.
