%%  Copyright (c) 2001 Dan Gudmundsson
%%
%%  See the file "license.terms" for information on usage and redistribution
%%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
%% 
%%     $Id: sdl_util.erl,v 1.4 2003/01/13 14:37:14 dgud Exp $
%%
%%%----------------------------------------------------------------------
%%% File    : sdl_util.erl
%%% Author  : Dan Gudmundsson <dgud@erix.ericsson.se>
%%% Purpose : 
%%% Created : 13 Sep 2000 by Dan Gudmundsson <dgud@erix.ericsson.se>
%%%----------------------------------------------------------------------

-module(sdl_util).
-author('dgud@erix.ericsson.se').

-compile(export_all).
%%-export([Function/Arity, ...]).

-include("sdl_video.hrl").
-include("sdl_util.hrl").
-include("gl.hrl").
-include("esdl.hrl").

-import(sdl, [call/1, cast/1]).

-define(SDL_UTIL_copySdlImage2GLArray, ?SDL_UTIL_HRL + 1).
-define(SDL_UTIL_debug, ?SDL_UTIL_HRL + 2).

-define(MYGL_malloc, ?SDL_UTIL_HRL + 3).
-define(MYGL_free,   ?SDL_UTIL_HRL + 4).
-define(MYGL_read,   ?SDL_UTIL_HRL + 5).
-define(MYGL_write,  ?SDL_UTIL_HRL + 6).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% EXTRA DRIVER: MEMORY HANDLING FUNCTIONS
%% These functions messes up the erlang idea
%% so be careful, it introduces the possibility to make 
%% pointer errors and memory leaks to your erlang program!
%%
%% But it simplifies the implementation for me :-(

%% Func:    malloc 
%% Args:    Size and type (?GL_UNSIGNED_INT, ...)
%% Returns: A reference to the memory area
%% Desc:    Allocates Size memory of Type
%%          where Type is one of the opengl types described in gl.hrl
malloc(Size, Type) ->
    <<Ptr:32>> = call( <<(?MYGL_malloc):16,Size:32, Type:16>>),
    #sdlmem{ptr = Ptr, type = Type, size =Size}.

%% Func:    free
%% Args:    Ref to memory
%% Desc:    Free memory allocated with malloc/2
free(Ref) ->
    cast( <<(?MYGL_free):16, (Ref#sdlmem.ptr):32>>).    

%% Func:    read/readBin
%% Args:    MemoryRef, Size
%% Returns: A list/Binary of Size elements of type Type
%% Desc:    Read allocated memory 
read(Ref, Size) ->
    Type = Ref#sdlmem.type,
    Bin = call( <<(?MYGL_read):16,Type:16, Size:32, (Ref#sdlmem.ptr):32>>),
    %% wb(binary_to_list(Bin)),
    buildlist(Size, Type, Bin, []).

readBin(Ref, Size) ->
    Type = Ref#sdlmem.type,
    call( <<(?MYGL_read):16,Type:16, Size:32, (Ref#sdlmem.ptr):32>>).


%% Func:    write
%% Args:    MemoryRef, List|Binary
%% Returns: ok
%% Desc:    Write to data memory 
write(Ref, List) when list(List) ->
    Size  = if 
		length(List) > Ref#sdlmem.size ->
		    Ref#sdlmem.size;
		true ->
		    length(List)
	    end,
    call( <<(?MYGL_write):16,(Ref#sdlmem.type):16, Size:32, (Ref#sdlmem.ptr):32,
	 (buildbin(List, Size, Ref#sdlmem.type))/binary>>),
    ok;
write(R, Bin) when binary(Bin) ->
    S = ?gl_type_size(R#sdlmem.type) /4, % Nr bytes
    case S * R#sdlmem.size of 
	Max when Max > size(Bin) ->
	    call( <<(?MYGL_write):16,1:16, (size(Bin)):32, (R#sdlmem.ptr):32, 
		 Bin/binary>>);
	Else -> 
	    exit({badmem, {maxSize, Else}})
    end,
    ok.	

%% Func:    buildbin
%% Args:    ListOfNumbers (or tuple), [MaxNo,] Type 
%% Returns: Binary
%% Desc:    Converts the first MaxNo numbers of ListOfNumbers to a 
%%          binary of the type Type.
%%          Where Type is GL_TYPE existing in gl.hrl (e.g. ?GL_DOUBLE)
buildbin(Bin, Type) when binary(Bin) ->    Bin;
buildbin(List1, Type) ->
    List = if
	       list(List1)  -> List1;
	       tuple(List1) -> tuple_to_list(List1)
	   end,
    case Type of 
	?GL_BYTE -> 
	    list_to_binary(List);
	?GL_UNSIGNED_BYTE ->
	    list_to_binary(List);
	_ ->
	    LoBs = buildbin2(List, Type, []),
	    list_to_binary(lists:reverse(LoBs))
    end.

buildbin(Bin, N, Type) when binary(Bin) ->
    Bin;
buildbin(List1, undefined, Type) ->
    buildbin(List1, Type);
buildbin(List1, N, Type) ->
    NL = if
	     list(List1)  -> List1;
	     tuple(List1) -> tuple_to_list(List1)
	 end,
    buildbin(lists:sublist(NL, N),Type).

buildbin2([], Type, Acc) ->    Acc ;
buildbin2([H|R], ?GL_FLOAT, Acc) ->
    buildbin2(R, ?GL_FLOAT, [<<H:32/float>> | Acc]);
buildbin2([H|R], ?GL_DOUBLE, Acc) ->
    buildbin2(R, ?GL_DOUBLE,[<<H:64/float>>| Acc]);
buildbin2([H|R], ?GL_UNSIGNED_INT, Acc) ->
    buildbin2(R, ?GL_UNSIGNED_INT, [<<H:32/unsigned>> | Acc]);
buildbin2([H|R], ?GL_INT, Acc) ->
    buildbin2(R, ?GL_INT, [<<H:32/signed>> | Acc]);
buildbin2([H|R], ?GL_SHORT, Acc) ->
    buildbin2(R, ?GL_SHORT, [<<H:16>> | Acc]);
buildbin2([H|R], ?GL_UNSIGNED_SHORT, Acc) ->
    buildbin2(R, ?GL_UNSIGNED_SHORT, [<<H:16/unsigned>> | Acc]);
buildbin2([H|R], ?GL_2_BYTES, Acc) ->
    buildbin2(R, ?GL_2_BYTES, [<<H:16/unsigned>> | Acc]);
buildbin2([H|R], ?GL_3_BYTES, Acc) ->
    buildbin2(R, ?GL_3_BYTES, [<<H:24/unsigned>> | Acc]);
buildbin2([H|R], ?GL_4_BYTES, Acc) ->
    buildbin2(R, ?GL_4_BYTES, [<<H:32/unsigned>> | Acc]);
buildbin2([true|R], boolean, Acc) ->
    buildbin2(R, boolean, [<<?GL_TRUE:8/unsigned>> | Acc]);
buildbin2([false|R], boolean, Acc) ->
    buildbin2(R, boolean, [<<?GL_FALSE:8/unsigned>> | Acc]);
buildbin2([H|R], boolean, Acc) ->
    buildbin2(R, boolean, [<<H:8/unsigned>> | Acc]).


%% Func:    buildlist
%% Args:    No, Type, Binary
%% Returns: ListOfNumbers
%% Desc:    Converts the first 'No' of numbers in the binary Binary  
%%          to a list of numbers of type Type.

buildlist(Type, Bin) ->
    buildlist(size(Bin), Type, Bin, []).
buildlist(undefined, Type, Bin) ->
    buildlist(size(Bin), Type, Bin, []);
buildlist(N, Type, Bin) ->
    buildlist(N, Type, Bin, []).

buildlist(_, _, <<>>, Acc) ->
    lists:reverse(Acc);
buildlist(0, _, _, Acc) ->
    lists:reverse(Acc);
buildlist(N, ?GL_UNSIGNED_INT, <<H:32/unsigned, Rest/binary>>, A)->
    buildlist(N -1, ?GL_UNSIGNED_INT, Rest, [H|A]);    
buildlist(N, ?GL_INT, <<H:32/signed, Rest/binary>>, A)->
    buildlist(N -1, ?GL_INT, Rest, [H|A]);    
buildlist(N, ?GL_UNSIGNED_SHORT, <<H:16/unsigned, Rest/binary>>, A)->
    buildlist(N -1, ?GL_UNSIGNED_SHORT, Rest, [H|A]);    
buildlist(N, ?GL_SHORT, <<H:16/signed, Rest/binary>>, A)->
    buildlist(N -1, ?GL_SHORT, Rest, [H|A]);    
buildlist(N, ?GL_UNSIGNED_BYTE, <<H:8/unsigned, Rest/binary>>, A)->
    buildlist(N -1, ?GL_UNSIGNED_BYTE, Rest, [H|A]);    
buildlist(N, ?GL_BYTE, <<H:8/signed, Rest/binary>>, A)->
    buildlist(N -1, ?GL_BYTE, Rest, [H|A]);    
buildlist(N, ?GL_FLOAT, <<H:32/float, Rest/binary>>, A)->
    buildlist(N -1, ?GL_FLOAT, Rest, [H|A]);    
buildlist(N, ?GL_DOUBLE, <<H:64/float, Rest/binary>>, A)->
    buildlist(N -1, ?GL_DOUBLE, Rest, [H|A]);    
buildlist(N, boolean, <<H:8/unsigned, Rest/binary>>, A)->
    buildlist(N -1, boolean, Rest, [(H /= ?GL_FALSE) |A]).     

%% Func:    term2bin
%% Args:    ListOfNumbers (or tuple), [MaxNo,] Type 
%% Returns: Binary
%% Desc:    Converts the first MaxNo numbers of ListOfNumbers to a 
%%          binary of the type Type.
%%          Where Type is GL_TYPE existing in gl.hrl (e.g. ?GL_DOUBLE)
term2bin(Bin, Type) when binary(Bin) ->    Bin;
term2bin(List1, Type) ->
    List = if
	       list(List1)  -> List1;
	       tuple(List1) -> tuple_to_list(List1)
	   end,
    case Type of 
	?GL_BYTE -> 
	    list_to_binary(List);
	?GL_UNSIGNED_BYTE ->
	    list_to_binary(List);
	_ ->
	    LoBs = term2bin2(List, Type, []),
	    list_to_binary(lists:reverse(LoBs))
    end.

term2bin(Bin, N, Type) when binary(Bin) ->
    Bin;
term2bin(List1, undefined, Type) ->
    term2bin(List1, Type);
term2bin(List1, N, Type) ->
    NL = if
	     list(List1)  -> List1;
	     tuple(List1) -> tuple_to_list(List1)
	 end,
    term2bin(lists:sublist(NL, N),Type).

term2bin2([], Type, Acc) ->    Acc ;
term2bin2([H|R], ?GL_FLOAT, Acc) ->
    term2bin2(R, ?GL_FLOAT, [<<H:32/float-native>> | Acc]);
term2bin2([H|R], ?GL_DOUBLE, Acc) ->
    term2bin2(R, ?GL_DOUBLE,[<<H:64/float-native>>| Acc]);
term2bin2([H|R], ?GL_UNSIGNED_INT, Acc) ->
    term2bin2(R, ?GL_UNSIGNED_INT, [<<H:32/unsigned-native>> | Acc]);
term2bin2([H|R], ?GL_INT, Acc) ->
    term2bin2(R, ?GL_INT, [<<H:32/signed-native>> | Acc]);
term2bin2([H|R], ?GL_SHORT, Acc) ->
    term2bin2(R, ?GL_SHORT, [<<H:16/signed-native>> | Acc]);
term2bin2([H|R], ?GL_UNSIGNED_SHORT, Acc) ->
    term2bin2(R, ?GL_UNSIGNED_SHORT, [<<H:16/unsigned-native>> | Acc]);
term2bin2([H|R], ?GL_2_BYTES, Acc) ->
    term2bin2(R, ?GL_2_BYTES, [<<H:16/unsigned-native>> | Acc]);
term2bin2([H|R], ?GL_3_BYTES, Acc) ->
    term2bin2(R, ?GL_3_BYTES, [<<H:24/unsigned-native>> | Acc]);
term2bin2([H|R], ?GL_4_BYTES, Acc) ->
    term2bin2(R, ?GL_4_BYTES, [<<H:32/unsigned-native>> | Acc]);
term2bin2([true|R], boolean, Acc) ->
    term2bin2(R, boolean, [<<?GL_TRUE:8/unsigned-native>> | Acc]);
term2bin2([false|R], boolean, Acc) ->
    term2bin2(R, boolean, [<<?GL_FALSE:8/unsigned-native>> | Acc]);
term2bin2([H|R], boolean, Acc) ->
    term2bin2(R, boolean, [<<H:8/unsigned-native>> | Acc]).


%% Func:    bin2list
%% Args:    No, Type, Binary
%% Returns: ListOfNumbers
%% Desc:    Converts the first 'No' of numbers in the binary Binary  
%%          to a list of numbers of type Type.

bin2list(Type, Bin) ->
    bin2list(size(Bin), Type, Bin, []).
bin2list(undefined, Type, Bin) ->
    bin2list(size(Bin), Type, Bin, []);
bin2list(N, Type, Bin) ->
    bin2list(N, Type, Bin, []).

bin2list(_, _, <<>>, Acc) ->
    lists:reverse(Acc);
bin2list(0, _, _, Acc) ->
    lists:reverse(Acc);
bin2list(N, ?GL_UNSIGNED_INT, <<H:32/unsigned-native, Rest/binary>>, A)->
    bin2list(N -1, ?GL_UNSIGNED_INT, Rest, [H|A]);    
bin2list(N, ?GL_INT, <<H:32/signed-native, Rest/binary>>, A)->
    bin2list(N -1, ?GL_INT, Rest, [H|A]);    
bin2list(N, ?GL_UNSIGNED_SHORT, <<H:16/unsigned-native, Rest/binary>>, A)->
    bin2list(N -1, ?GL_UNSIGNED_SHORT, Rest, [H|A]);    
bin2list(N, ?GL_SHORT, <<H:16/signed-native, Rest/binary>>, A)->
    bin2list(N -1, ?GL_SHORT, Rest, [H|A]);    
bin2list(N, ?GL_UNSIGNED_BYTE, <<H:8/unsigned-native, Rest/binary>>, A)->
    bin2list(N -1, ?GL_UNSIGNED_BYTE, Rest, [H|A]);    
bin2list(N, ?GL_BYTE, <<H:8/signed-native, Rest/binary>>, A)->
    bin2list(N -1, ?GL_BYTE, Rest, [H|A]);    
bin2list(N, ?GL_FLOAT, <<H:32/float-native, Rest/binary>>, A)->
    bin2list(N -1, ?GL_FLOAT, Rest, [H|A]);    
bin2list(N, ?GL_DOUBLE, <<H:64/float-native, Rest/binary>>, A)->
    bin2list(N -1, ?GL_DOUBLE, Rest, [H|A]);    
bin2list(N, boolean, <<H:8/unsigned-native, Rest/binary>>, A)->
    bin2list(N -1, boolean, Rest, [(H /= ?GL_FALSE) |A]).     

%% Func:    copySdlImage2GLArray
%% Args:    SdlImageRef, malloced Mem, BytesPerPixel
%% Returns: ok or exits
%% Desc:    Converts sdl_surface data to a byte RGB(A) buffer.
%%          sdl_surfaces are BGR and upside/down.
copySdlImage2GLArray(Image, Mem, Bpp) when record(Image, sdl_surface) ->
    copySdlImage2GLArray(Image#sdl_surface.self, Mem, Bpp);
copySdlImage2GLArray({surfacep, Image}, Mem, Bpp) 
  when Bpp == 3; Bpp == 4, record(Mem, sdlmem) ->
    <<1:8>> = call( 
		   <<(?SDL_UTIL_copySdlImage2GLArray):16,Image:32, (Mem#sdlmem.ptr):32, Bpp:8>>),
    ok.

%% Func: Debug 
%% Args: Level (0 is off)
%% Returns: ok
%% Desc: Set ESDL debug level
debug(L) when integer(L) ->
    cast(<<(?SDL_UTIL_debug):16, L:8>>).

%% Debug functions
wb([]) -> ok;
wb([A, B, C, D|R]) ->
    whex([A, B, C,D]),
    wb(R).

whex([]) -> io:format(" ~n");
whex([A|R]) ->
    A1 = A div 16,
    A2 = A rem 16,
    whex(A1),whex(A2),
    io:format(" "),
    whex(R);

whex(A) when A < 10 ->
    io:format("~p", [A]);
whex(A) when A == 10 ->
    io:format("A");
whex(A) when A == 11 ->
    io:format("B");
whex(A) when A == 12 ->
    io:format("C");
whex(A) when A == 13 ->
    io:format("D");
whex(A) when A == 14 ->
    io:format("E");
whex(A) when A == 15 ->
    io:format("F").
